#! /usr/bin/env python
import datetime
import json

import pytz
from dateutil.parser import parse
from threatminer import ThreatMiner

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandThreatMiner(Command):
    """
    # ThreatMiner

    https://www.threatminer.org/

    * Get report related to an IOC : `harpoon threatminer report DOMAIN`
    * Get samples related to a domain or IP : `harpoon threatminer sample DOMAIN`
    * Get metadata of a file : `harpoon threatminer metadata HASH`
    * Get HTTP traffic for a file : `harpoon threatminer traffic HASH`
    * Get hosts associated with a file : `harpoon threatminer host HASH`
    * Get mutants associated with a file : `harpoon threatminer mutant HASH`
    * Get AV detections associated with a file : `harpoon threatminer av HASH`
    * Get whois info on a domain : `harpoon threatminer whois DOMAIN`
    * Get Passive DNS info on a domain : `harpoon threatminer dns DOMAIN`
    * Get URLs related to a domain : `harpoon threatminer uri DOMAIN`
    * Get subdomains related to a domain : `harpoon threatminer subdomain DOMAIN`
    """

    name = "threatminer"
    description = "Requests TreatMiner database https://www.threatminer.org/"
    config = {"ThreatMiner": []}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "report", help="Request information a Threat Report"
        )
        parser_a.add_argument("INDICATOR", help="Indicator to check")
        parser_a.set_defaults(subcommand="report")
        parser_b = subparsers.add_parser(
            "sample", help="Get samples associated with a domain or IP"
        )
        parser_b.add_argument("INDICATOR", help="Indicator to check")
        parser_b.set_defaults(subcommand="sample")
        parser_c = subparsers.add_parser(
            "metadata", help="Get metadata associated with a file"
        )
        parser_c.add_argument("HASH", help="Hash to get metadata on")
        parser_c.set_defaults(subcommand="metadata")
        parser_d = subparsers.add_parser(
            "traffic", help="Get HTTP traffic associated with a file"
        )
        parser_d.add_argument("HASH", help="Hash to get metadata on")
        parser_d.set_defaults(subcommand="traffic")
        parser_e = subparsers.add_parser(
            "host", help="Get hosts associated with a file"
        )
        parser_e.add_argument("HASH", help="Hash to get info on")
        parser_e.set_defaults(subcommand="host")
        parser_f = subparsers.add_parser(
            "mutant", help="Get mutants associated with a file"
        )
        parser_f.add_argument("HASH", help="Hash to get info on")
        parser_f.set_defaults(subcommand="mutant")
        parser_g = subparsers.add_parser(
            "av", help="Get AV detections associated with a file"
        )
        parser_g.add_argument("HASH", help="Hash to get info on")
        parser_g.set_defaults(subcommand="av")
        parser_h = subparsers.add_parser("whois", help="Get whois info on a domain")
        parser_h.add_argument("DOMAIN", help="Domain")
        parser_h.set_defaults(subcommand="whois")
        parser_i = subparsers.add_parser("dns", help="Get Passive DNS info on a domain")
        parser_i.add_argument("DOMAIN", help="Domain")
        parser_i.set_defaults(subcommand="dns")
        parser_j = subparsers.add_parser("uri", help="Get URLs related to a domain")
        parser_j.add_argument("DOMAIN", help="Domain")
        parser_j.set_defaults(subcommand="uri")
        parser_k = subparsers.add_parser(
            "subdomain", help="Get subdomains related to a domain"
        )
        parser_k.add_argument("DOMAIN", help="Domain")
        parser_k.set_defaults(subcommand="subdomain")
        self.parser = parser

    def run(self, args, plugins):
        # FIXME : migrate to subcommands
        if "subcommand" in args:
            tm = ThreatMiner()
            if args.subcommand == "report":
                response = tm.get_report(unbracket(args.INDICATOR))
                if response["status_code"] == "200":
                    if len(response["results"]) > 0:
                        print("Reports found:")
                        for r in response["results"]:
                            print(
                                "{} {} - {}".format(r["year"], r["filename"], r["URL"])
                            )
                    else:
                        print("No report found for this indicator")
                elif response["status_code"] == "404":
                    print("No report found for this indicator")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "sample":
                response = tm.get_related_samples(unbracket(args.INDICATOR))
                if response["status_code"] == "200":
                    if len(response["results"]) > 0:
                        print("Samples found:")
                        for r in response["results"]:
                            print(r)
                    else:
                        print("No report found for this indicator")
                elif response["status_code"] == "404":
                    print("No report found for this indicator")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "metadata":
                response = tm.get_metadata(args.HASH)
                if response["status_code"] == "200":
                    for r in response["results"]:
                        for d in r:
                            print("{} - {}".format(d, r[d]))
                        print("")
                elif response["status_code"] == "404":
                    print("No report found for this indicator")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "traffic":
                response = tm.get_http_traffic(args.HASH)
                if response["status_code"] == "200":
                    for t in response["results"][0]["http_traffic"]:
                        for d in t.keys():
                            if d != "raw":
                                print("{} - {}".format(d, t[d]))
                        print("")
                elif response["status_code"] == "404":
                    print("No traffic found for this file")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "host":
                response = tm.get_hosts(args.HASH)
                if response["status_code"] == "200":
                    print("domains:")
                    for d in response["results"][0]["domains"]:
                        print("{} - {}".format(d["domain"], d["ip"]))
                    print("\nHosts:")
                    for h in response["results"][0]["hosts"]:
                        print(h)
                elif response["status_code"] == "404":
                    print("File not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "mutant":
                response = tm.get_mutants(args.HASH)
                if response["status_code"] == "200":
                    for m in response["results"][0]["mutants"]:
                        print(m)
                elif response["status_code"] == "404":
                    print("File not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "av":
                response = tm.get_av_detections(args.HASH)
                if response["status_code"] == "200":
                    for m in response["results"][0]["av_detections"]:
                        print(m)
                elif response["status_code"] == "404":
                    print("File not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "whois":
                response = tm.who_is(args.DOMAIN)
                if response["status_code"] == "200":
                    print(json.dumps(response["results"][0]["whois"], indent=4))
                elif response["status_code"] == "404":
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "dns":
                response = tm.passive_dns(args.DOMAIN)
                if response["status_code"] == "200":
                    for r in response["results"]:
                        print(
                            "{} - {} - {}".format(
                                r["ip"] if "ip" in r else r["domain"],
                                r["first_seen"],
                                r["last_seen"],
                            )
                        )
                elif response["status_code"] == "404":
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "uri":
                response = tm.get_uris(args.DOMAIN)
                if response["status_code"] == "200":
                    print(json.dumps(response["results"][0]["whois"], indent=4))
                elif response["status_code"] == "404":
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            elif args.subcommand == "subdomain":
                response = tm.get_subdomains(args.DOMAIN)
                if response["status_code"] == "200":
                    for s in response["results"]:
                        print(s)
                elif response["status_code"] == "404":
                    print("Domain not found")
                else:
                    print("Request failed: {}".format(response["status_message"]))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_domain(self, query, data):
        print("[+] Checking ThreatMiner...")
        tm = ThreatMiner()
        response = tm.passive_dns(query)
        if response["status_code"] == "200":
            for r in response["results"]:
                data["passive_dns"].append(
                    {
                        "ip": r["ip"],
                        "first": parse(r["first_seen"]).astimezone(pytz.utc),
                        "last": parse(r["last_seen"]).astimezone(pytz.utc),
                        "source": "ThreatMiner",
                    }
                )
        response = tm.get_report(query)
        if response["status_code"] == "200":
            for r in response["results"]:
                data["reports"].append(
                    {
                        "date": datetime.datetime(int(r["year"]), 1, 1),
                        "title": r["filename"],
                        "url": r["URL"],
                        "source": "ThreatMiner",
                    }
                )
        else:
            print(
                "Request to ThreatMiner failed: {}".format(response["status_message"])
            )
        response = tm.get_related_samples(query)
        if response["status_code"] == "200":
            for r in response["results"]:
                data["malware"].append(
                    {"hash": r, "date": None, "source": "ThreatMiner"}
                )

    def intel_ip(self, query, data):
        print("[+] Checking ThreatMiner...")
        tm = ThreatMiner()
        response = tm.passive_dns(query)
        if response["status_code"] == "200":
            for r in response["results"]:
                data["passive_dns"].append(
                    {
                        "domain": r["domain"],
                        "first": parse(r["first_seen"]).astimezone(pytz.utc),
                        "last": parse(r["last_seen"]).astimezone(pytz.utc),
                        "source": "ThreatMiner",
                    }
                )
        response = tm.get_report(query)
        if response["status_code"] == "200":
            for r in response["results"]:
                data["reports"].append(
                    {
                        "date": datetime.datetime(int(r["year"]), 1, 1),
                        "title": r["filename"],
                        "url": r["URL"],
                        "source": "ThreatMiner",
                    }
                )
        response = tm.get_related_samples(query)
        if response["status_code"] == "200":
            for r in response["results"]:
                data["malware"].append(
                    {"hash": r, "date": None, "source": "ThreatMiner"}
                )

    def intel_hash(self, query, data):
        print("[+] Checking ThreatMiner...")
        tm = ThreatMiner()
        response = tm.get_report(query)
        if response["status_code"] == "200":
            if len(response["results"]) > 0:
                for r in response["results"]:
                    data["reports"].append(
                        {
                            "date": datetime.datetime(int(r["year"]), 1, 1),
                            "title": r["filename"],
                            "url": r["URL"],
                            "source": "ThreatMiner",
                        }
                    )



#! /usr/bin/env python
import json

from pypermacc import Permacc, PermaccError

from harpoon.commands.base import Command


class CommandPemacc(Command):
    """
    # Perma.cc plugin

    **Query the permacc API**

    * Save an url : harpoon permacc save URL
    * See last 10 urls saved : harpoon permacc list
    * Download an archive for an url: harpoon permacc download Y6JJ-TDUJ
    * Get information on an archive : harpoon permacc info Y6JJ-TDUJ
    """

    name = "permacc"
    description = "Request Perma.cc information through the API"
    config = {"Permacc": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("save", help="Save an url in perma.cc")
        parser_a.add_argument("URL", help="Url")
        parser_a.set_defaults(subcommand="save")
        parser_b = subparsers.add_parser(
            "info", help="Show information on a saved page"
        )
        parser_b.add_argument("GUID", help="Guid (like Y6JJ-TDUJ)")
        parser_b.set_defaults(subcommand="info")
        parser_c = subparsers.add_parser("download", help="Download the WARC archive")
        parser_c.add_argument("GUID", help="Guid (like Y6JJ-TDUJ)")
        parser_c.add_argument(
            "--output", "-o", default="webpage.warc", help="Name of the file downloaded"
        )
        parser_c.set_defaults(subcommand="download")
        parser_d = subparsers.add_parser(
            "list", help="List pages archived by your account"
        )
        parser_d.set_defaults(subcommand="list")
        self.parser = parser

    def run(self, args, plugins):
        permacc = Permacc(self._config_data["Permacc"]["key"])
        if "subcommand" in args:
            if args.subcommand == "save":
                try:
                    saved = permacc.archive_create(args.URL)
                    print(
                        "Saved: https://perma.cc/%s - %s"
                        % (saved["guid"], saved["guid"])
                    )
                except PermaccError:
                    print("Failed")
            elif args.subcommand == "download":
                warc = permacc.archive_download(args.GUID)
                if args.output == "webpage.warc":
                    outputname = args.GUID + ".warc"
                else:
                    outputname = args.output
                with open(outputname, "wb") as f:
                    f.write(warc)
                print("Archive saved as %s" % outputname)
            elif args.subcommand == "info":
                infos = permacc.archive_detail(args.GUID)
                print(json.dumps(infos, sort_keys=False, indent=4))
            elif args.subcommand == "list":
                archives = permacc.user_archives()
                print("Last 10 pages:")
                for o in archives["objects"]:
                    print("- https://perma.cc/%s %s" % (o["guid"], o["url"]))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python3
import json
import logging

from dateutil.parser import parse
from greynoise import GreyNoise

from harpoon.commands.base import Command


class GreynoiseError(Exception):
    pass


class CommandGreyNoise(Command):
    """
    # GreyNoise

    See https://developer.greynoise.io/

    * To use GreyNoise Community (Free) API, set api_type = "community" in config.
    * Commuinty API only supports IP lookup command

    * List tags: `harpoon greynoise -l` (default output as json)
    * Search for an IP: `harpoon greynoise -i IP`
    * Run a GNQL query: `harpoon greynoise -q "classification:malicious tags:'emotet'"`
    """

    name = "greynoise"
    description = "Request information from GreyNoise API"
    config = {"GreyNoise": ["key", "api_type"]}

    def add_arguments(self, parser):
        parser.add_argument("--ip", "-i", help="Query an IP address")
        parser.add_argument("--list", "-l", help="List tags", action="store_true")
        parser.add_argument(
            "--query",
            "-q",
            help="Run a gnql query. Example: \"classification:malicious tags:'emotet'\" ",
        )
        parser.add_argument(
            "--format",
            "-f",
            help="Output format",
            choices=["csv", "json"],
            default="json",
        )
        self.parser = parser

    def print_results(self, res, args):
        if args.format == "json":
            print(json.dumps(res, indent=4, sort_keys=True))
        else:
            for k, v in res.items():
                print(k, ",", v)
        return

    def run(self, args, plugins):
        logging.getLogger("greynoise").setLevel(logging.CRITICAL)
        if self._config_data["GreyNoise"]["api_type"].lower() == "community":
            gn = GreyNoise(
                api_key=self._config_data["GreyNoise"]["key"],
                integration_name="Harpoon (https://github.com/Te-k/harpoon)",
                offering="community",
            )
        else:
            gn = GreyNoise(
                api_key=self._config_data["GreyNoise"]["key"],
                integration_name="Harpoon (https://github.com/Te-k/harpoon)",
            )
        if args.ip:
            res = gn.ip(args.ip)
            self.print_results(res, args)
        elif args.query:
            res = gn.query(args.query)
            self.print_results(res, args)
        elif args.list:
            res = gn.metadata()
            self.print_results(res, args)
        else:
            self.parser.print_help()

    def intel_ip(self, query, data):
        print("[+] Checking GreyNoise...")
        logging.getLogger("greynoise").setLevel(logging.CRITICAL)
        if self._config_data["GreyNoise"]["api_type"].lower() == "community":
            gn = GreyNoise(
                api_key=self._config_data["GreyNoise"]["key"],
                integration_name="Harpoon (https://github.com/Te-k/harpoon)",
                offering="community",
            )
            res = gn.ip(query)
            if res["noise"]:
                data["reports"].append(
                    {
                        "url": "https://viz.greynoise.io/ip/{}".format(query),
                        "title": "Seen by GreyNoise as {}".format(res["name"]),
                        "date": parse(res["last_seen"]) if "last_seen" in res else "",
                        "source": "GreyNoise",
                    }
                )
        else:
            gn = GreyNoise(
                api_key=self._config_data["GreyNoise"]["key"],
                integration_name="Harpoon (https://github.com/Te-k/harpoon)",
            )
            res = gn.ip(query)
            if res["seen"]:
                data["reports"].append(
                    {
                        "url": "https://viz.greynoise.io/ip/{}".format(query),
                        "title": "Seen by GreyNoise as {}".format(
                            ", ".join(res["tags"])
                        ),
                        "date": parse(res["last_seen"]) if "last_seen" in res else "",
                        "source": "GreyNoise",
                    }
                )



import requests


class UrlScanError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = message


class UrlScanQuotaExceeded(UrlScanError):
    pass


class UrlScan(object):
    def __init__(self, key=None):
        self.url = "https://urlscan.io/api/v1/"
        self.api_key = key

    def _get(self, url, params):
        headers = {
            "User-Agent": "phishtank/Harpoon (https://github.com/Te-k/harpoon)",
            "Content-Type": "application/json"
        }
        if self.api_key:
            headers["API-Key"] = self.api_key
        r = requests.get(url, params=params, headers=headers)
        if r.status_code != 200:
            if r.status_code == 429:
                if "message" in r.json():
                    raise UrlScanQuotaExceeded(r.json()["message"])
                else:
                    raise UrlScanQuotaExceeded("Quota exceeded")
            else:
                raise UrlScanError(
                    "Invalid HTTP Code returned: {}".format(r.status_code))
        res = r.json()
        if "status" in res:
            if res['status'] == 429:
                raise UrlScanQuotaExceeded(res["message"])
        return res

    def search(self, query, size=100, offset=0):
        params = {
            'q': query,
        }
        return self._get(self.url + "search/", params)

    def view(self, uid):
        return self._get(self.url + "result/" + uid, {})

    def quota(self):
        return self._get("https://urlscan.io/user/quotas/", {})



#! /usr/bin/env python
import json

from spyonweb import (SpyOnWeb, SpyOnWebError, SpyOnWebInvalidToken,
                      SpyOnWebNotFound)

from harpoon.commands.base import Command


class CommandSpyonweb(Command):
    """
    # Spy On Web

    **Queries Spy On Web database http://spyonweb.com/**

    * Search for a domain: `harpoon spyonweb domain example.org`
    * Search for an adsense id: `harpoon spyonweb adsense ID`
    * Search for a Google Analytics id: `harpoon spyonweb analytics ID`
    * Search for an IP address: `harpoon spyonweb ip IP`
    """

    name = "spyonweb"
    description = "Search in SpyOnWeb through the API"
    config = {"SpyOnWeb": ["token"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommands")
        parser_b = subparsers.add_parser("domain", help="Query a domain")
        parser_b.add_argument("DOMAIN", help="Domain to be requested")
        parser_b.add_argument(
            "--raw", "-r", action="store_true", help="Print raw list of domains"
        )
        parser_b.set_defaults(subcommand="domain")
        parser_c = subparsers.add_parser("adsense", help="Query an adsense id")
        parser_c.add_argument("ID", help="id to be requested")
        parser_c.add_argument(
            "--raw", "-r", action="store_true", help="Print raw list of domains"
        )
        parser_c.set_defaults(subcommand="adsense")
        parser_d = subparsers.add_parser(
            "analytics", help="Query a Google Analytics id"
        )
        parser_d.add_argument("ID", help="id to be requested")
        parser_d.add_argument(
            "--raw", "-r", action="store_true", help="Print raw list of domains"
        )
        parser_d.set_defaults(subcommand="analytics")
        parser_e = subparsers.add_parser("ip", help="Query an IP Address")
        parser_e.add_argument("IP", help="IP address to be requested")
        parser_e.add_argument(
            "--raw", "-r", action="store_true", help="Print raw list of domains"
        )
        parser_e.set_defaults(subcommand="ip")
        parser_f = subparsers.add_parser("nsdomain", help="Query an Name Server domain")
        parser_f.add_argument("DOMAIN", help="Name Server Domain to be requested")
        parser_f.add_argument(
            "--raw", "-r", action="store_true", help="Print raw list of domains"
        )
        parser_f.set_defaults(subcommand="nsdomain")
        parser_g = subparsers.add_parser("nsip", help="Query a Name Server IP address")
        parser_g.add_argument("IP", help="Name Server IP Address to be requested")
        parser_g.add_argument(
            "--raw", "-r", action="store_true", help="Print raw list of domains"
        )
        parser_g.set_defaults(subcommand="nsip")
        self.parser = parser

    def run(self, args, plugins):
        if hasattr(args, "subcommand"):
            s = SpyOnWeb(self._config_data["SpyOnWeb"]["token"])
            if args.subcommand == "domain":
                try:
                    res = s.summary(args.DOMAIN)
                except SpyOnWebNotFound:
                    print("Domain not found")
                except SpyOnWebInvalidToken:
                    print("Invalid configuration")
                except SpyOnWebError as e:
                    print("Weird error: %s" % e.message)
                else:
                    if args.raw:
                        print(json.dumps(res, sort_keys=True, indent=4))
                    else:
                        print("--------------- %s -----------------" % args.DOMAIN)
                        if "ip" in res:
                            print("IP:")
                            for i in res["ip"]:
                                print("-%s: %i entries" % (i, res["ip"][i]))
                        if "adsense" in res:
                            print("AdSense:")
                            for i in res["adsense"]:
                                print("-%s: %i entries" % (i, res["adsense"][i]))
                        if "analytics" in res:
                            print("Analytics:")
                            for i in res["analytics"]:
                                print("-%s: %i entries" % (i, res["analytics"][i]))
                        if "dns_servers" in res:
                            print("DNS Servers:")
                            for i in res["dns_servers"]:
                                print("-%s: %i entries" % (i, res["dns_servers"][i]))
            elif args.subcommand == "adsense":
                try:
                    res = s.adsense(args.ID)
                except SpyOnWebNotFound:
                    print("Adsense id not found")
                except SpyOnWebInvalidToken:
                    print("Invalid configuration")
                except SpyOnWebError as e:
                    print("Weird error: %s" % e.message)
                else:
                    if args.raw:
                        for i in res["items"]:
                            print(i)
                    else:
                        print("--------------- %s -----------------" % args.ID)
                        print(
                            "Fetched %i domains over %i"
                            % (res["fetched"], res["found"])
                        )
                        for i in res["items"]:
                            print("-%s (%s)" % (i, res["items"][i]))
            elif args.subcommand == "analytics":
                try:
                    res = s.analytics(args.ID)
                except SpyOnWebNotFound:
                    print("Analytic id not found")
                except SpyOnWebInvalidToken:
                    print("Invalid configuration")
                except SpyOnWebError as e:
                    print("Weird error: %s" % e.message)
                else:
                    if args.raw:
                        for i in res["items"]:
                            print(i)
                    else:
                        print("--------------- %s -----------------" % args.ID)
                        print(
                            "Fetched %i domains over %i"
                            % (res["fetched"], res["found"])
                        )
                        for i in res["items"]:
                            print("-%s (%s)" % (i, res["items"][i]))
            elif args.subcommand == "ip":
                try:
                    res = s.ip(args.IP)
                except SpyOnWebNotFound:
                    print("IP address not found")
                except SpyOnWebInvalidToken:
                    print("Invalid configuration")
                except SpyOnWebError as e:
                    print("Weird error: %s" % e.message)
                else:
                    if args.raw:
                        for i in res["items"]:
                            print(i)
                    else:
                        print("--------------- %s -----------------" % args.IP)
                        print(
                            "Fetched %i domains over %i"
                            % (res["fetched"], res["found"])
                        )
                        for i in res["items"]:
                            print("-%s (%s)" % (i, res["items"][i]))
            elif args.subcommand == "nsdomain":
                try:
                    res = s.nameserver_domain(args.DOMAIN)
                except SpyOnWebNotFound:
                    print("Name Server domain not found")
                except SpyOnWebInvalidToken:
                    print("Invalid configuration")
                except SpyOnWebError as e:
                    print("Weird error: %s" % e.message)
                else:
                    if args.raw:
                        for i in res["items"]:
                            print(i)
                    else:
                        print("--------------- %s -----------------" % args.DOMAIN)
                        print(
                            "Fetched %i domains over %i"
                            % (res["fetched"], res["found"])
                        )
                        for i in res["items"]:
                            print("-%s (%s)" % (i, res["items"][i]))
            elif args.subcommand == "nsip":
                try:
                    res = s.nameserver_ip(args.IP)
                except SpyOnWebNotFound:
                    print("Name Server IP not found")
                except SpyOnWebInvalidToken:
                    print("Invalid configuration")
                except SpyOnWebError as e:
                    print("Weird error: %s" % e.message)
                else:
                    if args.raw:
                        for i in res["items"]:
                            print(i)
                    else:
                        print("--------------- %s -----------------" % args.IP)
                        print(
                            "Fetched %i domains over %i"
                            % (res["fetched"], res["found"])
                        )
                        for i in res["items"]:
                            print("-%s (%s)" % (i, res["items"][i]))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python3
import json

import pytz
from dateutil.parser import parse

from harpoon.commands.base import Command
from harpoon.lib.urlhaus import UrlHaus, UrlHausError
from harpoon.lib.utils import unbracket


class CommandUrlhaus(Command):
    """
    # URLhaus

    Query https://urlhaus.abuse.ch/ API

    * Get information on an IP : `harpoon urlhaus get-host 111.174.128.51`
    * Get information on a hash : `harpoon urlhaus get-payload [HASH]`
    * Download a sample : `harpoon urlhaus get-payload [HASH]`
    """

    name = "urlhaus"
    description = "Request urlhaus.abuse.ch API"
    config = {"UrlHaus": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("get-url", help="Request info on a URL")
        parser_a.add_argument("url", help="url")
        parser_a.set_defaults(subcommand="get-url")

        parser_b = subparsers.add_parser(
            "get-host",
            help="Request info on a host: IPv4 address, hostname or domain name",
        )
        parser_b.add_argument("host", help="host")
        parser_b.set_defaults(subcommand="get-host")

        parser_c = subparsers.add_parser(
            "get-payload", help="Request info about a payload: md5 or sha256"
        )
        parser_c.add_argument("payload", help="payload")
        parser_c.set_defaults(subcommand="get-payload")

        parser_d = subparsers.add_parser(
            "get-tag",
            help="Request info about a tag: Gozi, Trickbot...",
        )
        parser_d.add_argument("tag", help="tag")
        parser_d.set_defaults(subcommand="get-tag")

        parser_d = subparsers.add_parser(
            "get-signature", help="Request info about a signature: Gozi, Trickbot"
        )
        parser_d.add_argument("signature", help="signature")
        parser_d.set_defaults(subcommand="get-signature")

        parser_e = subparsers.add_parser(
            "get-sample", help="Request a malware sample identified by a hash (sha256)"
        )
        parser_e.add_argument("hash", help="hash")
        parser_e.set_defaults(subcommand="get-sample")

        self.parser = parser

    def run(self, args, plugins):
        urlhaus = UrlHaus(self._config_data["UrlHaus"]["key"])
        if "subcommand" in args:
            try:
                if args.subcommand == "get-url":
                    res = urlhaus.get_url(args.url)
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "get-host":
                    res = urlhaus.get_host(unbracket(args.host))
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "get-payload":
                    res = urlhaus.get_payload(args.payload)
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "get-tag":
                    res = urlhaus.get_tag(args.tag)
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "get-signature":
                    res = urlhaus.get_signature(args.signature)
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "get-sample":
                    data = urlhaus.get_sample(args.hash)
                    if data:
                        with open(args.hash, "wb") as f:
                            f.write(data)
                        print("Sample saved as {}".format(args.hash))
                    else:
                        print("Sample not found")
                else:
                    self.parser.print_help()
            except UrlHausError:
                print("UrlHaus : query failed ¯\\_(ツ)_/¯")
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type in ["domain", "ip"]:
            print("[+] Checking URLHaus...")
            try:
                urlhaus = UrlHaus(self._config_data["UrlHaus"]["key"])
                res = urlhaus.get_host(unbracket(query))
            except UrlHausError:
                print("Error with the query")
            else:
                if "urls" in res:
                    for r in res["urls"]:
                        data["urls"].append(
                            {
                                "date": parse(r["date_added"]).astimezone(pytz.utc),
                                "url": r["url"],
                                "ip": "",
                                "source": "UrlHaus",
                            }
                        )
        elif type == "hash":
            print("[+] Checking URLHaus...")
            try:
                urlhaus = UrlHaus(self._config_data["UrlHaus"]["key"])
                res = urlhaus.get_payload(query)
            except UrlHausError:
                print("Error with the query")
            else:
                if "urlhaus_download" in res:
                    data["samples"].append(
                        {
                            "date": parse(res["firstseen"]),
                            "source": "UrlHaus",
                            "url": res["urlhaus_download"],
                        }
                    )
                if "urls" in res:
                    for r in res["urls"]:
                        data["urls"].append(
                            {
                                "date": parse(r["firstseen"])
                                if "firstseen" in r
                                else "",
                                "url": r["url"],
                                "link": r["urlhaus_reference"],
                                "source": "UrlHaus",
                            }
                        )



#! /usr/bin/env python
import json

import shodan
from dateutil.parser import parse

from harpoon.commands.base import Command, Subcommand
from harpoon.lib.utils import unbracket


class SubcommandIP(Subcommand):
    description = "Get information on an IP address"
    cmd = "ip"

    def add_arguments(self, parser):
        parser.add_argument("IP", help="IP to be searched")
        parser.add_argument(
            "--history",
            "-H",
            action="store_true",
            help="Also display historical information",
        )
        parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            help="Verbose mode (display raw json)",
        )
        parser.add_argument(
            "-s",
            "--summary",
            action="store_true",
            help="Only display information for ports 22, 80 and 443",
        )

    def run(self, args):
        api = shodan.Shodan(self._config_data["Shodan"]["key"])
        try:
            self.data = api.host(args.IP, history=args.history)
        except shodan.exception.APIError:
            self.data = None

    def display(self, args):
        if self.data is None:
            print("IP not found in Shodan")
            return

        if args.verbose:
            print(json.dumps(self.data, indent=4))
        else:
            if args.summary:
                for d in self.data["data"]:
                    if d["port"] == 22:
                        print(
                            "{} - port 22 ssh - {}".format(
                                d["timestamp"][:19], d["data"].split("\n")[0]
                            )
                        )
                    elif d["port"] == 80:
                        print(
                            '{} - port 80 http - Server "{}"'.format(
                                d["timestamp"][:19], d["http"]["server"]
                            )
                        )
                    elif d["port"] == 443:
                        if "cert" in d["ssl"]:
                            print(
                                '%s - port 443 https - Cert "%s" "%s" %s - Server "%s"'
                                % (
                                    d["timestamp"][:19],
                                    d["ssl"]["cert"]["subject"]["CN"],
                                    d["ssl"]["cert"]["issuer"]["CN"],
                                    d["ssl"]["cert"]["fingerprint"]["sha1"],
                                    d["http"]["server"],
                                )
                            )
                        else:
                            print(
                                '%s - port 443 https - Cert Unknown- Server "%s"'
                                % (d["timestamp"][:19], d["http"]["server"])
                            )
            else:
                for d in self.data["data"]:
                    print(d["timestamp"])
                    print(d["_shodan"]["module"])
                    print("%s/%i" % (d["transport"], d["port"]))
                    print(d["data"])
                    if "html" in d:
                        print(d["html"][:2000])
                    if "http" in d:
                        print(json.dumps(d["http"])[:3000])
                    print("")


class SubcommandSearch(Subcommand):
    description = "Search in shodan"
    cmd = "search"

    def add_arguments(self, parser):
        parser.add_argument("QUERY", help="Query")
        parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            help="Verbose mode (display raw json)",
        )

    def run(self, args):
        api = shodan.Shodan(self._config_data["Shodan"]["key"])
        self.data = api.search(args.QUERY)

    def display(self, args):
        if args.verbose:
            print(json.dumps(self.data, indent=4))
        else:
            print("%i results" % self.data["total"])
            for r in self.data["matches"]:
                print(
                    "[+] {} ({}): port {}/{} -> {}\n".format(
                        r["ip_str"],
                        r["org"],
                        r["transport"],
                        r["port"],
                        r["data"][:1000],
                    )
                )


class SubcommandSsh(Subcommand):
    description = "Get history of ssh keys from Shodan historical data"
    cmd = "ssh"

    def add_arguments(self, parser):
        parser.add_argument("IP", help="IP address")

    def run(self, args):
        api = shodan.Shodan(self._config_data["Shodan"]["key"])
        try:
            res = api.host(unbracket(args.IP), history=True)
        except shodan.exception.APIError as e:
            print(e)
            self.data = None
            return

        self.data = {}
        for event in res["data"]:
            if event["_shodan"]["module"] == "ssh":
                if "ssh" not in event:
                    continue
                fingerprint = event["ssh"]["fingerprint"]
                date = parse(event["timestamp"])
                if fingerprint not in self.data:
                    self.data[fingerprint] = {
                        "first": date,
                        "last": date,
                        "fingerprint": fingerprint,
                    }
                else:
                    if self.data[fingerprint]["first"] > date:
                        self.data[fingerprint]["first"] = date
                    if self.data[fingerprint]["last"] < date:
                        self.data[fingerprint]["last"] = date

    def display(self, args):
        if self.data is None:
            print("IP not found in Shodan")
            return

        for val in sorted(self.data.values(), key=lambda x: x["first"]):
            print(
                "{} - {} -> {}".format(
                    val["fingerprint"],
                    val["first"].strftime("%Y-%m-%d"),
                    val["last"].strftime("%Y-%m-%d"),
                )
            )


class SubcommandQuota(Subcommand):
    description = "Show Shodan account quota information"
    cmd = "quota"

    def run(self, args):
        api = shodan.Shodan(self._config_data["Shodan"]["key"])
        self.data = api.info()

    def display(self, args):
        print(json.dumps(self.data, indent=4))


# ------------------------------------ Main Command ---------------------------
class CommandShodan(Command):
    """
    # Shodan

    **Queries information from shodan.io API***

    * Get information on an IP : `harpoon shodan ip IP`
    * Get summary (only ports 22, 80 and 443) of historical data on an ip : `harpoon shodan ip -H -s IP`
    * Get raw json of historical data : `harpoon shodan ip -H -v IP`
    * Search in the database: `harpoon shodan search SEARCH`
    """

    name = "shodan"
    description = "Requests Shodan API"
    config = {"Shodan": ["key"]}

    def __init__(self, config):
        super().__init__(config=config)
        self.add_subcommand(SubcommandIP)
        self.add_subcommand(SubcommandSearch)
        self.add_subcommand(SubcommandSsh)
        self.add_subcommand(SubcommandQuota)

    def intel_ip(self, query, data):
        print("[+] Checking Shodan...")
        api = shodan.Shodan(self._config_data["Shodan"]["key"])
        try:
            res = api.host(query)
        except shodan.exception.APIError:
            pass
        else:
            for p in res["ports"]:
                data["ports"].append({"port": p, "source": "Shodan", "info": ""})



class Subcommand(object):
    """
    Subcommand object that can be used by a Command
    """

    description = ""
    cmd = ""

    def __init__(self, conf):
        self.data = {}
        self._config_data = conf

    def add_arguments(self, parser):
        pass

    def run(self, args):
        raise NotImplementedError()

    def display(self, args):
        """
        Display data
        """
        pass


class Command(object):
    config = None  # Describes the configuration params
    update_needed = False

    def __init__(self, config):
        self._subcommands = {}
        self._config_data = config

    @property
    def config_needed(self):
        if self.config is None:
            return False
        if len(list(self.config.keys())) == 0:
            return False
        pname = list(self.config.keys())[0]
        return len(self.config[pname]) > 0

    def update(self):
        pass

    def add_subcommand(self, sc):
        """
        Adds a subcommand to the command
        """
        self._subcommands[sc.cmd] = sc(self._config_data)

    def test_config(self):
        """
        Test that the config params defined are in the conf file
        """
        if self.config is None:
            return True
        if len(self.config.keys()) == 0:
            return True
        else:
            pname = list(self.config.keys())[0]
            if pname not in self._config_data:
                if len(self.config[pname]) == 0:
                    return True
                else:
                    return False
            else:
                for d in self.config[pname]:
                    if d not in self._config_data[pname]:
                        return False
                    else:
                        if self._config_data[pname][d] == "":
                            return False
            return True

    def check_intel(self):
        """
        Check if intel is disabled in the configuration
        """
        if self.config is None:
            return True
        if len(self.config.keys()) == 0:
            return True
        else:
            pname = list(self.config.keys())[0]
            if pname not in self._config_data:
                return True
            else:
                if "intel" not in self._config_data[pname]:
                    return True
                else:
                    return self._config_data[pname]["intel"].lower() != "false"

    def intel(self, type, query, data):
        """
        Add information to the global intel command
        type : can be ip or domain (string)
        query : domain or ip address (string)
        data : contains data depending on the type
            For domains: passive_dns, urls, malware, files, reports
        """
        if type == "ip":
            self.intel_ip(query, data)
        elif type == "domain":
            self.intel_domain(query, data)
        elif type == "hash":
            self.intel_hash(query, data)
        elif type == "email":
            self.intel_email(query, data)

    def intel_ip(self, query, data):
        """
        Adds information to the global intel command
        """
        pass

    def intel_domain(self, query, data):
        """
        Adds information to the global intel command
        """
        pass

    def intel_hash(self, query, data):
        """
        Adds information to the global intel command
        """
        pass

    def intel_email(self, query, data):
        """
        Adds information to the global intel command
        """
        pass

    def add_arguments(self, parser):
        if len(self._subcommands) > 0:
            subparser = parser.add_subparsers(help="Subcommand")
            for sc in self._subcommands.values():
                p = subparser.add_parser(sc.cmd, help=sc.description)
                sc.add_arguments(p)
                p.set_defaults(subcommand=sc.cmd)
        self._parser = parser

    def run(self, args, plugins):
        """
        Run the command
        """
        if len(self._subcommands) > 0:
            if "subcommand" in args:
                if args.subcommand in self._subcommands.keys():
                    self._subcommands[args.subcommand].run(args)
                    self._subcommands[args.subcommand].display(args)
                else:
                    self._parser.print_help()
            else:
                self._parser.print_help()
        else:
            raise NotImplementedError()



#! /usr/bin/env python
from dns import exception, resolver, reversename

from harpoon.commands.base import Command
from harpoon.lib.utils import is_ip, unbracket


class CommandDns(Command):
    """
    # DNS plugin

    **Map DNS information for a domain or an IP address**

    * Check DNS information on a domain: `harpoon dns DOMAIN`
    * Check for all types of DNS entries: `harpoon dns -e DOMAIN`

    """

    # TODO: graph output
    # TODO: option to define DNS server
    name = "dns"
    description = "Map DNS information for a domain or an IP"
    config = {}
    all_types = [
        "NONE",
        "A",
        "NS",
        "MD",
        "MF",
        "CNAME",
        "SOA",
        "MB",
        "MG",
        "MR",
        "NULL",
        "WKS",
        "PTR",
        "HINFO",
        "MINFO",
        "MX",
        "TXT",
        "RP",
        "AFSDB",
        "X25",
        "ISDN",
        "RT",
        "NSAP",
        "NSAP-PTR",
        "SIG",
        "KEY",
        "PX",
        "GPOS",
        "AAAA",
        "LOC",
        "NXT",
        "SRV",
        "NAPTR",
        "KX",
        "CERT",
        "A6",
        "DNAME",
        "OPT",
        "APL",
        "DS",
        "SSHFP",
        "IPSECKEY",
        "RRSIG",
        "NSEC",
        "DNSKEY",
        "DHCID",
        "NSEC3",
        "NSEC3PARAM",
        "TLSA",
        "HIP",
        "CDS",
        "CDNSKEY",
        "CSYNC",
        "SPF",
        "UNSPEC",
        "EUI48",
        "EUI64",
        "TKEY",
        "TSIG",
        "IXFR",
        "AXFR",
        "MAILB",
        "MAILA",
        "ANY",
        "URI",
        "CAA",
        "TA",
        "DLV",
    ]

    def add_arguments(self, parser):
        parser.add_argument("TARGET", help="Domain or IP to query")
        parser.add_argument(
            "--extended",
            "-e",
            action="store_true",
            help="Extended testing of all DNS types",
        )
        self.parser = parser

    def owner_to_email(self, owner):
        rev = owner[:-1].split(".")
        return ".".join(rev[:-2]) + "@" + ".".join(rev[-2:])

    def run(self, args, plugins):
        if is_ip(unbracket(args.TARGET)):
            # That's an IP address
            ptr_n = str(reversename.from_address(unbracket(args.TARGET)))
            try:
                answer = [entry for entry in resolver.resolve(ptr_n, "PTR")][0]
                print("%s - %s" % (ptr_n, str(answer)))
            except (resolver.NXDOMAIN, resolver.NoAnswer):
                print("%s - %s" % (ptr_n, "NXDOMAIN"))
        else:
            cip = plugins["ip"]
            if args.extended:
                for a in self.all_types:
                    try:
                        answers = resolver.resolve(unbracket(args.TARGET), a)
                        for rdata in answers:
                            print(a, ":", rdata.to_text())
                    except Exception:
                        pass
            else:
                target = unbracket(args.TARGET)
                # A
                print("# A")
                try:
                    answers = resolver.resolve(target, "A")
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No A entry")
                except resolver.NoNameservers:
                    print("No A entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for rdata in answers:
                        info = cip.ipinfo(rdata.address)
                        print(
                            "%s: ASN%i %s - %s %s"
                            % (
                                rdata.address,
                                info["asn"],
                                info["asn_name"],
                                info["city"],
                                info["country"],
                            )
                        )

                # AA
                print("")
                print("# AAAA")
                try:
                    answers = resolver.resolve(target, "AAAA")
                    for rdata in answers:
                        print(rdata.address)
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No AAAA entry configured")
                except resolver.NoNameservers:
                    print("No AAAA entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")

                # DNS Servers
                print("\n# NS")
                try:
                    answers = resolver.resolve(target, "NS")
                except (resolver.NoAnswer, resolver.NXDOMAIN, resolver.NoNameservers):
                    # That's pretty unlikely
                    print("No NS entry configured")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for entry in answers:
                        ttarget = str(entry.target)
                        if is_ip(ttarget):
                            # Pretty unlikely
                            info = cip.ipinfo(ttarget)
                            print(
                                "%s - ASN%i %s - %s %s"
                                % (
                                    ttarget,
                                    info["asn"],
                                    info["asn_name"],
                                    info["city"],
                                    info["country"],
                                )
                            )
                        else:
                            try:
                                ip = [
                                    b.address for b in resolver.resolve(ttarget, "A")
                                ][0]
                            except resolver.NXDOMAIN:
                                # Hostname without IPv4
                                print(ttarget)
                            else:
                                # Hostname
                                info = cip.ipinfo(ip)
                                print(
                                    "%s - %s - ASN%i %s - %s %s"
                                    % (
                                        ttarget,
                                        ip,
                                        info["asn"],
                                        info["asn_name"],
                                        info["city"],
                                        info["country"],
                                    )
                                )

                # MX
                print("\n# MX:")
                try:
                    answers = resolver.resolve(target, "MX")
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No MX entry configured")
                except resolver.NoNameservers:
                    print("No MX entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for rdata in answers:
                        if is_ip(rdata.exchange):
                            # IP directly
                            info = cip.ipinfo(rdata.exchange)
                            print(
                                "%i %s - ASN%i %s - %s %s"
                                % (
                                    rdata.preference,
                                    rdata.exchange,
                                    info["asn"],
                                    info["asn_name"],
                                    info["city"],
                                    info["country"],
                                )
                            )
                        else:
                            try:
                                ip = [
                                    b.address
                                    for b in resolver.resolve(rdata.exchange, "A")
                                ][0]
                            except (resolver.NoAnswer, resolver.NXDOMAIN):
                                # Hostname without IPv4
                                print("%i %s" % (rdata.preference, rdata.exchange))
                            else:
                                # Hostname
                                info = cip.ipinfo(ip)
                                print(
                                    "%i %s - %s - ASN%i %s - %s %s"
                                    % (
                                        rdata.preference,
                                        rdata.exchange,
                                        ip,
                                        info["asn"],
                                        info["asn_name"],
                                        info["city"],
                                        info["country"],
                                    )
                                )

                # SOA
                print("\n# SOA")
                try:
                    answers = resolver.resolve(target, "SOA")
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No SOA entry configured")
                except resolver.NoNameservers:
                    print("No SOA entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    entry = [b for b in answers][0]
                    print("NS: %s" % str(entry.mname))
                    print("Owner: %s" % self.owner_to_email(str(entry.rname)))

                # TXT
                print("\n# TXT:")
                try:
                    answers = resolver.resolve(target, "TXT")
                except (resolver.NoAnswer, resolver.NXDOMAIN):
                    print("No TXT entry configured")
                except resolver.NoNameservers:
                    print("No TXT entry: SERVFAIL")
                except exception.Timeout:
                    print("Timeout")
                else:
                    for a in answers:
                        print(a.to_text())



import json

import requests


class PithusError(Exception):
    def __init__(self, message, **kwargs):
        Exception.__init__(self, message)
        self.message = message


class PithusQuotaExceeded(PithusError):
    pass


class Pithus(object):
    def __init__(self, config=None):
        self.url = config['url']
        self.api_key = config['key']
        self.headers = {
            "User-Agent": "Harpoon (https://github.com/Te-k/harpoon)",
        }
        if self.api_key:
            self.headers["Authorization"] = "Token " + self.api_key
        else:
            PithusError(
                "Missing token, visit beta.pithus.org/hunting to retrieve it")

    def handle_request(self, r):
        if r.status_code == 200:
            return r.json()
        elif r.status_code == 404:
            raise PithusError("Report not found")
        elif r.status_code == 453:
            raise PithusQuotaExceeded("Quota exceeded")
        elif r.status_code != 200 or r.status_code != 453:
            raise PithusError(r)
        else:
            raise PithusError(
                'Invalid HTTP Code returned: {}'.format(r.status_code))

    def handle_error(self, error_code):
        if error_code == 2:
            return "Success"
        elif error_code == -1:
            return "Error"
        elif error_code == 1:
            return "Running"
        else:
            return "Unknow status"

    def pretty_print(self, result, result_type):
        print("######## Pithus results")
        print("")

        if result_type == 'search':
            for res in result:
                print("[*] Result for " + self.url + "report/" +
                      res["source"]["sha256"])
                print("App name      ", res["source"]["app_name"])
                print("Handle        ", res["source"]["handle"])
                print("Sha256        ", res["source"]["sha256"])
                print("Frosted       ", res["source"]
                      ["frosting_data"]["is_frosted"])
                print("Uploaded at   ", res["source"]["uploaded_at"])
                print("Signed        ", res["source"]["is_signed"])
                if "quark" in res["source"]:
                    print("Quark result  ", res["source"]["quark"]["threat_level"])

                if res["score"] is not None:
                    print("Score:         ", res["score"])

                if "highlight" in res:
                    if len(res["highlight"]) > 0:
                        print("")
                        print("---- Search matches:")
                        for matches_key in res["highlight"]:
                            for value in res["highlight"][matches_key]:
                                print("{} - {}".format(matches_key, value.replace("<mark>", "").replace("</mark>", "")))

                if len(res["source"]["features"]) > 0:
                    print("")
                    print("---- Features:")
                    for feat in res["source"]["features"]:
                        print(feat)

                if "vt" in res["source"]:
                    print("")
                    print("----- VirusTotal:")
                    print("VT malicious   ", res["source"]["vt"]["malicious"])
                    print("VT suspicious  ", res["source"]["vt"]["suspicious"])
                    print("VT undetected  ", res["source"]["vt"]["undetected"])

                print("")
                print("")

        elif result_type == 'status':
            print("APKiD analysis status            ",
                  self.handle_error(result['apkid_analysis']))
            print("SSDeep analsysis status          ",
                  self.handle_error(result['ssdeep_analysis']))
            print("Class extracted                  ",
                  self.handle_error(result['extract_classes']))
            print("Quark analysis status            ",
                  self.handle_error(result['quark_analysis']))
            print("MobSF analysis status            ",
                  self.handle_error(result['mobsf_analysis']))
            print("VirusTotal analsysis status      ",
                  self.handle_error(result['vt_analysis']))
            print("Malware Bazaar analsysis status  ",
                  self.handle_error(result['malware_bazaar_analysis']))
            print("")
            print("Analysis date                    ", result['analysis_date'])

        elif result_type == 'report':
            print("--------- General information:")
            print("Handle             ", result['handle'])
            print("App name           ", result['app_name'])
            print("Version name       ", result['version_name'])
            print("Version code       ", result['version_code'])
            print("Uploaded at        ", result['uploaded_at'])
            print("Main activity      ", result['main_activity'])
            print("Target SDK version ", result["target_sdk_version"])
            print("Frosted            ", result["frosting_data"]["is_frosted"])
            print("")

            if len(result['certificates']) > 0:
                print("--------- Certificates:")
                for certificate in result['certificates']:
                    print(json.dumps(certificate, indent=4))
                print("")

            if len(result['activities']) > 0:
                print("--------- Activities:")
                for activity in result['activities']:
                    print(json.dumps(activity, indent=4))
                print("")

            if len(result['permissions']) > 0:
                print("-------- Permissions:")
                for permission in result['permissions']:
                    print(json.dumps(permission, indent=4))
                print("")

            if len(result['providers']) > 0:
                print("------- Providers:")
                for provider in result['providers']:
                    print(json.dumps(provider, indent=4))
                print("")

            if len(result['services']) > 0:
                print("------ Services:")
                for service in result['services']:
                    print(json.dumps(service, indent=4))
                print("")

            if len(result['domains_analysis']) > 0:
                print("------- Domains found:")
                for domain in result['domains_analysis']:
                    print(json.dumps(domain['_name'], indent=4))
                print("")

        else:
            raise PithusError("Something went wrong... Try again, maybe?")

    def _get(self, url, result_type):
        r = requests.get(url, headers=self.headers)
        return self.handle_request(r)
        # return self.pretty_print(url, res, result_type)

    def _post(self, url, params, result_type):
        r = requests.post(url, json=params, headers=self.headers)
        res = self.handle_request(r)
        return self.pretty_print(url, res, result_type)

    def report(self, query):
        return self._get(self.url + "report/" + query, 'report')

    def status(self, query):
        return self._get(self.url + "status/" + query, 'status')

    def search(self, query):
        params = {
            "q": query,
        }
        return self._post(self.url + "search/", params, 'search')

    def upload(self, data):
        params = {
            "file": data,
        }
        url = self.url + "/upload"
        res = requests.post(url, files=params, headers=self.headers)

        if res.status_code == 200:
            print("Upload successful!")
            print(self.url + "/report/" +
                  res.json()['file_sha256'])
        else:
            print(res.status_code)



import json

import requests


class MalShareFailed(Exception):
    pass


class MalShareNotFound(MalShareFailed):
    pass


class MalShareSampleMissing(MalShareFailed):
    pass


class MalShare(object):
    def __init__(self, key):
        self.key = key
        self.base_url = "https://malshare.com/api.php"
        self.ua = "Harpoon"

    def _request(self, params):
        params['api_key'] = self.key
        headers = {'User-Agent': self.ua}
        r = requests.get(
            self.base_url,
            params=params,
            headers=headers
        )
        if r.status_code == 200:
            return r
        else:
            if r.status_code == 404:
                raise MalShareNotFound()
            raise MalShareFailed()

    def list_last24h_samples(self):
        params = {'action': 'getlistraw'}
        r = self._request(params)
        return r.text.strip().split("\n")

    def list_last24h_sources(self):
        params = {'action': 'getsourcesraw'}
        r = self._request(params)
        return r.strip().split("\n")

    def download(self, hash):
        """
        Download file
        """
        data = self._request({'action': 'getfile', 'hash': hash})
        if "Sample not found" in data.text:
            raise MalShareNotFound()
        else:
            if "Error => Sample Missing":
                raise MalShareSampleMissing()
            else:
                return data.text

    def file_info(self, hash):
        """
        Get information on the file
        """
        data = self._request({'action': 'details', 'hash': hash})
        if "Sample not found" in data.text:
            raise MalShareNotFound()
        else:
            return data.json()

    def search(self, query):
        data = self._request({'action': 'search', 'query': query})
        if data.text == '':
            return {}
        else:
            return [json.loads(a) for a in data.text.strip().split('\n')]



#! /usr/bin/env python
import json

import phonenumbers
import requests

from harpoon.commands.base import Command


class CommandNumVerify(Command):
    """
    # Numverify.com plugin

    **Query NumVerify API**

    * Check a phone number
    """

    name = "numverify"
    description = "Query phone number information from NumVerify"
    config = {"NumVerify": ["key"]}

    def query(self, phone, cc, key):
        params = {"access_key": key, "number": phone, "country_code": cc, "format": 1}
        r = requests.get("http://apilayer.net/api/validate", params=params)
        return r.json()

    def add_arguments(self, parser):
        parser.add_argument("PHONE", help="Phone number")
        parser.add_argument("COUNTRY", help="Country code (two letters)")
        self.parser = parser

    def run(self, args, plugins):
        x = phonenumbers.parse(args.PHONE, args.COUNTRY)
        print(
            "Requesting phone +{} - {} ({})".format(
                x.country_code, x.national_number, args.COUNTRY
            )
        )
        res = self.query(
            x.national_number, args.COUNTRY, self._config_data["NumVerify"]["key"]
        )
        print(json.dumps(res, sort_keys=False, indent=4))



#! /usr/bin/env python
import os
import sys
import traceback

from harpoon.commands.base import Command
from harpoon.lib.utils import is_ip, unbracket


class CommandIntel(Command):
    """
    # Intel

    Gather information from multiple Threat Intelligence platforms

    * **harpoon intel domain DOMAIN**
    """

    name = "intel"
    description = "Gather information on a domain"
    config = None
    geocity = os.path.join(
        os.path.expanduser("~"), ".config/harpoon/GeoLite2-City.mmdb"
    )
    geoasn = os.path.join(os.path.expanduser("~"), ".config/harpoon/GeoLite2-ASN.mmdb")
    asnname = os.path.join(os.path.expanduser("~"), ".config/harpoon/asnnames.csv")
    asncidr = os.path.join(os.path.expanduser("~"), ".config/harpoon/asncidr.dat")

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "domain", help="Gather Threat Intelligence information on a domain"
        )
        parser_a.add_argument("DOMAIN", help="Domain")
        parser_a.add_argument(
            "--all",
            "-a",
            action="store_true",
            help="Query all plugins configured and available",
        )
        parser_a.set_defaults(subcommand="domain")
        parser_b = subparsers.add_parser(
            "ip", help="Gather Threat Intelligence information on an IP address"
        )
        parser_b.add_argument("IP", help="IP address")
        parser_b.add_argument(
            "--all",
            "-a",
            action="store_true",
            help="Query all plugins configured and available",
        )
        parser_b.set_defaults(subcommand="ip")

        parser_c = subparsers.add_parser(
            "hash", help="Gather Threat Intelligence information on a hash"
        )
        parser_c.add_argument("HASH", help="Hash")
        parser_c.add_argument(
            "--all",
            "-a",
            action="store_true",
            help="Query all plugins configured and available",
        )
        parser_c.set_defaults(subcommand="hash")

        parser_d = subparsers.add_parser(
            "email", help="Gather Threat Intelligence information on an email address"
        )
        parser_d.add_argument("EMAIL", help="Email address")
        parser_d.add_argument(
            "--all",
            "-a",
            action="store_true",
            help="Query all plugins configured and available",
        )
        parser_d.set_defaults(subcommand="email")
        # parser_d = subparsers.add_parser(
        # "subdomain", help="Gather Threat Intelligence information on subdomains of a given domain"
        # )

        self.parser = parser

    def print_threat_report(self, reports):
        if len(reports) > 0:
            print("----------------- Intelligence Report")
            for report in reports:
                print(
                    "{} - {} - {} - {}".format(
                        report["date"].strftime("%Y-%m-%d") if report["date"] else "",
                        report["title"],
                        report["url"],
                        report["source"],
                    )
                )
            print("")

    def print_passive_dns(self, passive_dns, domain=True):
        if len(passive_dns) > 0:
            print("----------------- Passive DNS")
            for r in sorted(passive_dns, key=lambda x: x["first"], reverse=True):
                if domain:
                    el = r["domain"]
                else:
                    el = r["ip"]
                print(
                    "[+] %-40s (%s -> %s)(%s)"
                    % (
                        el,
                        r["first"].strftime("%Y-%m-%d"),
                        r["last"].strftime("%Y-%m-%d") if r["last"] else "",
                        r["source"],
                    )
                )
            print("")

    def print_malware(self, malware):
        if len(malware) > 0:
            print("----------------- Malware")
            for r in malware:
                print(
                    "[%s] %s %s"
                    % (
                        r["source"],
                        r["hash"],
                        r["date"].strftime("%Y-%m-%d") if r["date"] else "",
                    )
                )
            print("")

    def print_files(self, files):
        if len(files) > 0:
            print("----------------- Files")
            for r in files:
                if r["date"] != "":
                    print(
                        "[%s] %s (%s)"
                        % (
                            r["source"],
                            r["hash"],
                            r["date"].strftime("%Y-%m-%d"),
                        )
                    )
                else:
                    print(
                        "[%s] %s"
                        % (
                            r["source"],
                            r["hash"],
                        )
                    )
            print("")

    def print_urls(self, urls):
        if len(urls) > 0:
            print("----------------- Urls")
            for r in sorted(urls, key=lambda x: x["date"], reverse=True):
                print(
                    "{:9} {} - {} {}".format(
                        "[" + r["source"] + "]",
                        r["url"],
                        r["ip"],
                        r["date"].strftime("%Y-%m-%d"),
                    )
                )
            print("")

    def do_intel_domain(self, domain, args, plugins):
        data = {
            "passive_dns": [],
            "urls": [],
            "malware": [],
            "files": [],
            "reports": [],
            "subdomains": [],
        }
        print("###################### %s ###################" % domain)
        for p in plugins:
            try:
                if args.all:
                    if plugins[p].test_config():
                        plugins[p].intel("domain", domain, data)
                else:
                    if plugins[p].test_config() and plugins[p].check_intel():
                        plugins[p].intel("domain", domain, data)
            except Exception:
                print("Command {} failed".format(p))
                traceback.print_exc()
        print("")

        self.print_threat_report(data["reports"])
        self.print_malware(data["malware"])
        self.print_files(data["files"])
        self.print_urls(data["urls"])
        self.print_passive_dns(data["passive_dns"], domain=False)

        if len(data["subdomains"]) > 0:
            print("-------------------- Subdomains")
            for r in data["subdomains"]:
                print("[%s] %s" % (r["source"], r["domain"]))

        if sum([len(data[b]) for b in data]) == 0:
            print("Nothing found")

    def do_intel_ip(self, ip, args, plugins):
        """
        Check intel on an IP address using other plugins
        """
        data = {
            "passive_dns": [],
            "urls": [],
            "malware": [],
            "files": [],
            "reports": [],
            "ports": [],
        }
        print("###################### %s ###################" % args.IP)
        for p in plugins:
            try:
                if args.all:
                    if plugins[p].test_config():
                        plugins[p].intel("ip", ip, data)
                else:
                    if plugins[p].test_config() and plugins[p].check_intel():
                        plugins[p].intel("ip", ip, data)
            except Exception:
                print("Command {} failed".format(p))
                traceback.print_exc()
        print("")

        self.print_threat_report(data["reports"])
        self.print_malware(data["malware"])
        self.print_files(data["files"])
        self.print_urls(data["urls"])
        self.print_passive_dns(data["passive_dns"])

        if len(data["ports"]) > 0:
            print("--------------------- Open Ports")
            for p in data["ports"]:
                print("{:6} - {} ({})".format(p["port"], p["info"], p["source"]))
            print("")

        if sum([len(data[b]) for b in data]) == 0:
            print("Nothing found")

    def do_intel_hash(self, hash_, args, plugins):
        """
        Query a hash to all plugins
        """
        data = {"samples": [], "urls": [], "network": [], "reports": []}
        print("############### {}".format(hash_))
        for p in plugins:
            try:
                if args.all:
                    if plugins[p].test_config():
                        plugins[p].intel("hash", hash_, data)
                else:
                    if plugins[p].test_config() and plugins[p].check_intel():
                        plugins[p].intel("hash", hash_, data)
            except Exception:
                print("Command {} failed".format(p))
                traceback.print_exc()
        print("")

        self.print_threat_report(data["reports"])

        if len(data["samples"]) > 0:
            print("----------------- Samples")
            for sample in data["samples"]:
                print(
                    "{} - {} {}".format(
                        sample["date"].strftime("%Y-%m-%d") if sample["date"] else "",
                        sample["source"],
                        sample["url"],
                    )
                )
                if "infos" in sample:
                    for info in sample["infos"]:
                        print("- {} - {}".format(info, sample["infos"][info]))
            print("")

        if len(data["network"]) > 0:
            print("------------------ Network")
            for host in data["network"]:
                if "host2" in host:
                    print(
                        "{:30} {} - {}".format(
                            "{} ({})".format(host["host"], host["host2"]),
                            host["source"],
                            host["url"],
                        )
                    )
                else:
                    print(
                        "{:30} {} - {}".format(
                            host["host"], host["source"], host["url"]
                        )
                    )
            print("")

        if len(data["urls"]) > 0:
            print("----------------- Urls")
            for report in data["urls"]:
                print(
                    "{} - {} - {}".format(
                        report["url"], report["link"], report["source"]
                    )
                )
            print("")

        if sum([len(data[b]) for b in data]) == 0:
            print("Nothing found")

    def do_intel_email(self, email, args, plugins):
        """
        Query intel on emails
        """
        data = {
            "domains": [],
            "keys": [],
            "mentions": [],
            # "samples": [],
            # "urls": [],
            # "reports": []
        }

        print("############### {}".format(email))
        for p in plugins:
            try:
                if args.all:
                    if plugins[p].test_config():
                        plugins[p].intel("email", email, data)
                else:
                    if plugins[p].test_config() and plugins[p].check_intel():
                        plugins[p].intel("email", email, data)
            except Exception:
                print("Command {} failed".format(p))
                traceback.print_exc()
        print("")

        # Print domains
        if len(data["domains"]) > 0:
            print("----------------- Domains")
            for d in data["domains"]:
                print(
                    "[%s] %-25s (%s)"
                    % (d["registered"], d["domain"], ", ".join(d["infos"]))
                )

        # Print keys
        if len(data["keys"]) > 0:
            print("----------------- Keys")
            # TODO: sort by date
            for k in data["keys"]:
                print(
                    "[+] %s\t%s\t%s %s"
                    % (k["id"], k["date"].strftime("%Y-%m-%d"), k["name"], k["email"])
                )

        # Print mentions
        if len(data["mentions"]) > 0:
            print("----------------- Mentions")
            for page in data["mentions"]:
                print("[{}] {}".format(page["source"], page["url"]))

        if sum([len(data[b]) for b in data]) == 0:
            print("Nothing found")

    def run(self, args, plugins):
        if "subcommand" in args:
            if args.subcommand == "domain":
                self.do_intel_domain(unbracket(args.DOMAIN), args, plugins)
            elif args.subcommand == "ip":
                if not is_ip(unbracket(args.IP)):
                    print("Invalid IP address")
                    sys.exit(1)
                self.do_intel_ip(unbracket(args.IP), args, plugins)
            elif args.subcommand == "hash":
                # TODO : check hash format
                self.do_intel_hash(args.HASH, args, plugins)
            elif args.subcommand == "email":
                self.do_intel_email(args.EMAIL, args, plugins)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import json

import pytz
from dateutil.parser import parse
from OTXv2 import IndicatorTypes, OTXv2

from harpoon.commands.base import Command
from harpoon.lib.utils import typeguess

OTX_TYPES = {
    "domain": IndicatorTypes.DOMAIN,
    "IPv4": IndicatorTypes.IPv4,
    "IPv6": IndicatorTypes.IPv6,
    "url": IndicatorTypes.URL,
    "md5": IndicatorTypes.FILE_HASH_MD5,
    "sha1": IndicatorTypes.FILE_HASH_SHA1,
    "sha256": IndicatorTypes.FILE_HASH_SHA256,
    "pehash": IndicatorTypes.FILE_HASH_PEHASH,
    "imphash": IndicatorTypes.FILE_HASH_IMPHASH,
    "cidr": IndicatorTypes.CIDR,
    "file_path": IndicatorTypes.FILE_PATH,
    "hostname": IndicatorTypes.HOSTNAME,
    "mutex": IndicatorTypes.MUTEX,
    "cve": IndicatorTypes.CVE,
}


class CommandOtx(Command):
    """
    # AlienVault Open Threat Exchange

    **Requests information from AlienVault Open Threat Exchange https://www.alienvault.com/open-threat-exchange**

    * Search for an indicator: `harpoon otx -s 127.0.0.1`
    * Search for a list of indicators from a file: `harpoon otx -f FILE`
    * Get information on a pulse: `harpoon otx -p PULSE_ID`

    You can get raw JSON output with `-j`, and gives the IOC type with `-t`
    """

    name = "otx"
    description = "Requests information from AlienVault OTX"
    config = {"AlienVaultOtx": ["key"]}

    def add_arguments(self, parser):
        # TODO: rewrite as subcommands
        parser.add_argument("--pulse", "-p", help="Event infos")
        parser.add_argument("--search", "-s", help="Search for indicators")
        parser.add_argument(
            "--file", "-f", help="Check from a list of indicators in a file"
        )
        parser.add_argument(
            "--raw", "-r", help="Print raw information", action="store_true"
        )
        parser.add_argument(
            "--json", "-j", help="Print json information", action="store_true"
        )
        parser.add_argument(
            "--type",
            "-t",
            help="Type for search",
            default="guess",
            choices=[
                "guess",
                "domain",
                "IPv4",
                "IPv6",
                "url",
                "md5",
                "sha1",
                "sha256",
                "pehash",
                "imphash",
                "cidr",
                "file_path",
                "hostname",
                "mutex",
                "cve",
            ],
        )
        self.parser = parser

    def run(self, args, plugins):
        otx = OTXv2(self._config_data["AlienVaultOtx"]["key"])
        if args.pulse:
            try:
                indicators = otx.get_pulse_indicators(args.pulse)
            except Exception:
                print("Pulse not found")
            else:
                if args.json:
                    print(json.dumps(indicators, sort_keys=True, indent=4))
                elif args.raw:
                    for i in indicators:
                        print(i["indicator"])
                else:
                    for i in indicators:
                        if len(i["type"]) < 5:
                            t = "\t\t\t"
                        else:
                            t = "\t\t"

                        print(
                            "[+] %s %s%s%s"
                            % (i["created"], i["type"], t, i["indicator"])
                        )
        elif args.search:
            if args.type == "guess":
                t = typeguess(args.search)
            else:
                t = args.type

            res = otx.get_indicator_details_full(OTX_TYPES[t], args.search)
            if args.json:
                print(json.dumps(res, sort_keys=True, indent=4))
            else:
                if "analysis" in res:
                    if res["analysis"]["analysis"]:
                        analysis = res["analysis"]["analysis"]
                        print("File analysed the %s" % analysis["datetime_int"])
                        if "info" in analysis:
                            print("Infos:")
                            for r in analysis["info"]["results"]:
                                print("\t%s : %s" % (r, analysis["info"]["results"][r]))
                        if "exiftool" in analysis["plugins"]:
                            print("Exiftool:")
                            for r in analysis["plugins"]["exiftool"]["results"]:
                                print(
                                    "\t%s : %s"
                                    % (r, analysis["plugins"]["exiftool"]["results"][r])
                                )
                        if "yarad" in analysis["plugins"]:
                            print("Yara detection:")
                            for r in analysis["plugins"]["yarad"]["results"][
                                "detection"
                            ]:
                                print("\t%s : %s" % (r["category"], r["rule_name"]))

                        print("")
                    else:
                        print("No analysis on this file")
                else:
                    print("No analysis on this file")
                if len(res["general"]["pulse_info"]["pulses"]) > 0:
                    print(
                        "Listed in %s pulses"
                        % len(res["general"]["pulse_info"]["pulses"])
                    )
                    for p in res["general"]["pulse_info"]["pulses"]:
                        print("\t-%s" % p["name"])
                        print("\t\t%s" % p["description"].replace("\n", " "))
                        print("\t\tCreated: %s" % p["created"])
                        print("\t\tReferences: %s" % ", ".join(p["references"]))
                        print("\t\tid: %s" % p["id"])
                else:
                    print("Not listed in any pulse")
                if "passive_dns" in res:
                    if len(res["passive_dns"]["passive_dns"]) > 0:
                        print("Passive DNS:")
                        for p in res["passive_dns"]["passive_dns"]:
                            print(
                                "\t%s%s on IP %s [%s -> %s]"
                                % (
                                    p["hostname"],
                                    p["indicator_link"],
                                    p["address"],
                                    p["first"],
                                    p["last"],
                                )
                            )
                        print("")
                if "url_list" in res:
                    if len(res["url_list"]["url_list"]) > 0:
                        print("URL list:")
                        for u in res["url_list"]["url_list"]:
                            if "result" in u:
                                if "urlworker" in u["result"]:
                                    if "ip" in u["result"]["urlworker"]:
                                        print(
                                            "\t[%s] %s on IP %s"
                                            % (
                                                u["date"],
                                                u["url"],
                                                u["result"]["urlworker"]["ip"],
                                            )
                                        )
                                    else:
                                        print("\t[%s] %s" % (u["date"], u["url"]))
                                else:
                                    print("\t[%s] %s" % (u["date"], u["url"]))
                            else:
                                print("\t[%s] %s" % (u["date"], u["url"]))
                        print("")
        elif args.file:
            with open(args.file, "r") as f:
                data = f.read().split("\n")
            for d in data:
                if d.strip() != "":
                    if args.type == "guess":
                        t = typeguess(d.strip())
                    else:
                        t = args.type

                    print("========== %s" % d.strip())
                    try:
                        res = otx.get_indicator_details_full(OTX_TYPES[t], d.strip())
                        if len(res["general"]["pulse_info"]["pulses"]) > 0:
                            print(
                                "Listed in %s pulses"
                                % len(res["general"]["pulse_info"]["pulses"])
                            )
                            for p in res["general"]["pulse_info"]["pulses"]:
                                print("\t-%s" % p["name"])
                                print("\t\t%s" % p["description"].replace("\n", " "))
                                print("\t\tCreated: %s" % p["created"])
                                print("\t\tReferences: %s" % ", ".join(p["references"]))
                                print("\t\tid: %s" % p["id"])
                        else:
                            print("Not listed in any pulse")
                    except OTXv2.BadRequest:
                        print("Request failed")
        else:
            self.parser.print_help()

    def intel_domain(self, query, data):
        print("[+] Checking OTX...")
        try:
            otx = OTXv2(self._config_data["AlienVaultOtx"]["key"])
            res = otx.get_indicator_details_full(IndicatorTypes.DOMAIN, query)
            for pulse in res["general"]["pulse_info"]["pulses"]:
                data["reports"].append(
                    {
                        "date": parse(pulse["created"]).astimezone(pytz.utc),
                        "title": pulse["name"],
                        "source": "OTX",
                        "url": "https://otx.alienvault.com/pulse/{}".format(
                            pulse["id"]
                        ),
                    }
                )
            # Get Passive DNS
            if "passive_dns" in res:
                for r in res["passive_dns"]["passive_dns"]:
                    data["passive_dns"].append(
                        {
                            "ip": r["hostname"],
                            "first": parse(r["first"]).astimezone(pytz.utc),
                            "last": parse(r["last"]).astimezone(pytz.utc),
                            "source": "OTX",
                        }
                    )
            if "url_list" in res:
                for r in res["url_list"]["url_list"]:
                    if "result" in r:
                        data["urls"].append(
                            {
                                "date": parse(r["date"]).astimezone(pytz.utc),
                                "url": r["url"],
                                "ip": r["result"]["urlworker"]["ip"]
                                if "ip" in r["result"]["urlworker"]
                                else "",
                                "source": "OTX",
                            }
                        )
                    else:
                        data["urls"].append(
                            {
                                "date": parse(r["date"]).astimezone(pytz.utc),
                                "url": r["url"],
                                "ip": "",
                                "source": "OTX",
                            }
                        )
            # Some pulses have domains as hostnames
            res = otx.get_indicator_details_full(IndicatorTypes.HOSTNAME, query)
            for pulse in res["general"]["pulse_info"]["pulses"]:
                data["reports"].append(
                    {
                        "date": parse(pulse["created"]).astimezone(pytz.utc),
                        "title": pulse["name"],
                        "source": "OTX",
                        "url": "https://otx.alienvault.com/pulse/{}".format(
                            pulse["id"]
                        ),
                    }
                )
        except AttributeError:
            print("OTX crashed  ¯\\_(ツ)_/¯")

    def intel_ip(self, query, data):
        print("[+] Checking OTX...")
        try:
            otx = OTXv2(self._config_data["AlienVaultOtx"]["key"])
            res = otx.get_indicator_details_full(IndicatorTypes.IPv4, query)
            for pulse in res["general"]["pulse_info"]["pulses"]:
                data["reports"].append(
                    {
                        "date": parse(pulse["created"]).astimezone(pytz.utc),
                        "title": pulse["name"],
                        "source": "OTX",
                        "url": "https://otx.alienvault.com/pulse/{}".format(
                            pulse["id"]
                        ),
                    }
                )
            # Get Passive DNS
            if "passive_dns" in res:
                for r in res["passive_dns"]["passive_dns"]:
                    data["passive_dns"].append(
                        {
                            "domain": r["hostname"],
                            "first": parse(r["first"]).astimezone(pytz.utc),
                            "last": parse(r["last"]).astimezone(pytz.utc),
                            "source": "OTX",
                        }
                    )
            if "url_list" in res:
                for r in res["url_list"]["url_list"]:
                    if "result" in r:
                        data["urls"].append(
                            {
                                "date": parse(r["date"]).astimezone(pytz.utc),
                                "url": r["url"],
                                "ip": r["result"]["urlworker"]["ip"]
                                if "ip" in r["result"]["urlworker"]
                                else "",
                                "source": "OTX",
                            }
                        )
                    else:
                        data["urls"].append(
                            {
                                "date": parse(r["date"]).astimezone(pytz.utc),
                                "url": r["url"],
                                "ip": "",
                                "source": "OTX",
                            }
                        )
        except AttributeError:
            print("OTX crashed  ¯\\_(ツ)_/¯")

    def intel_hash(self, query, data):
        print("[+] Checking OTX...")
        t = typeguess(query)
        try:
            otx = OTXv2(self._config_data["AlienVaultOtx"]["key"])
            res = otx.get_indicator_details_full(OTX_TYPES[t], query)
            for pulse in res["general"]["pulse_info"]["pulses"]:
                data["reports"].append(
                    {
                        "date": parse(pulse["created"]).astimezone(pytz.utc),
                        "title": pulse["name"],
                        "source": "OTX",
                        "url": "https://otx.alienvault.com/pulse/{}".format(
                            pulse["id"]
                        ),
                    }
                )
            if "analysis" in res:
                if "analysis" in res["analysis"]:
                    if "plugins" in res["analysis"]["analysis"]:
                        if "cuckoo" in res["analysis"]["analysis"]["plugins"]:
                            done = []
                            for d in res["analysis"]["analysis"]["plugins"]["cuckoo"][
                                "result"
                            ]["network"]["domains"]:
                                data["network"].append(
                                    {
                                        "source": "OTX",
                                        "url": "https://otx.alienvault.com/indicator/file/{}".format(
                                            query
                                        ),
                                        "host": d["domain"],
                                        "host2": d["ip"],
                                    }
                                )
                                done.append(d["ip"])
                                done.append(d["domain"])
                            for ip in res["analysis"]["analysis"]["plugins"]["cuckoo"][
                                "result"
                            ]["network"]["hosts"]:
                                if ip["ip"] not in done:
                                    data["network"].append(
                                        {
                                            "source": "OTX",
                                            "url": "https://otx.alienvault.com/indicator/file/{}".format(
                                                query
                                            ),
                                            "host": ip["ip"],
                                        }
                                    )
        except AttributeError:
            print("OTX crashed  ¯\\_(ツ)_/¯")



#! /usr/bin/env python
import json
import os
import sys

import pyasn
import requests

from harpoon.commands.base import Command


class CommandAsn(Command):
    """
    # ASN command

    **Get ASN information from different sources**

    * Offline ASN info: `harpoon asn info ASN15134`
    * ASN Info from peering db (https://peeringdb.com/) : `harpoon asn db 1228` (-j for raw output)
    * List subnets for an ASN: `harpoon asn subnet 15348`

    Example:
    ```
    $ harpoon asn db 18182
    Name: So-net Entertainment Taiwan Limited
    Website: http://www.so-net.net.tw
    Created: 2011-06-15T06:16:44Z
    ```
    """

    name = "asn"
    description = "Gather information on an ASN"
    config = None
    asn_name = os.path.join(os.path.expanduser("~"), ".config/harpoon/asnnames.csv")
    asncidr = os.path.join(os.path.expanduser("~"), ".config/harpoon/asncidr.dat")
    asncaida = os.path.join(os.path.realpath(__file__)[:-16], "data/caida.txt")

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("info", help="Information on an ASn number")
        parser_a.add_argument("ASN", help="ASN Number")
        parser_a.set_defaults(subcommand="info")
        parser_b = subparsers.add_parser(
            "db", help="ASN information from peeringdb (https://peeringdb.com/)"
        )
        parser_b.add_argument("ASN", help="ASN Number")
        parser_b.add_argument("--json", "-j", help="Show raw json", action="store_true")
        parser_b.set_defaults(subcommand="db")
        parser_c = subparsers.add_parser(
            "subnet", help="List of subnets for an ASN number"
        )
        parser_c.add_argument("ASN", help="ASN Number")
        parser_c.set_defaults(subcommand="subnet")
        self.parser = parser

    def check_update(self):
        """
        Check if files obtained through updates are on the system
        """
        if (
            not os.path.isfile(self.asn_name)
            or not os.path.isfile(self.asncidr)
            or not os.path.isfile(self.asncaida)
        ):
            print("ASN files not downloaded on the system")
            print("Please run harpoon update before using harpoon")
            sys.exit(1)

    def asn_caida(self, asn):
        """
        Read the 2015 CAIDA database and returns the classification for the given ASN
        input: asn : integer
        output {'source': SOURCE, 'type': TYPE}
        """
        with open(self.asncaida, "r") as f:
            line = f.readline()
            while line != "":
                if line.startswith("#"):
                    line = f.readline()
                    continue
                data = line.split("|")
                if int(data[0]) == asn:
                    return {"source": data[1], "type": data[2].strip()}
                line = f.readline()
        return {"source": "", "type": "Unknown"}

    def asnname(self, asn):
        """
        Search for the ASN name based on its number
        Input: asnnumber (integer)
        returns: string
        Returns an empty string if not found
        """
        # Search for name
        f = open(self.asn_name, "r")
        line = f.readline()
        while line != "":
            s = line.split("|")
            if s[0] == str(asn):
                f.close()
                return s[1].strip()
            line = f.readline()
        f.close()
        return ""

    def run(self, args, plugins):
        if hasattr(args, "ASN"):
            if args.ASN.lower().startswith("asn"):
                asn = int(args.ASN[3:])
            elif args.ASN.lower().startswith("as"):
                asn = int(args.ASN[2:])
            else:
                asn = int(args.ASN)
        else:
            self.parser.print_help()
            sys.exit(0)
        if "subcommand" in args:
            if args.subcommand == "info":
                self.check_update()
                info = self.asnname(asn)
                if len(info):
                    print("ASN%i - %s" % (asn, info))
                else:
                    print("Unknown ASN")
            elif args.subcommand == "subnet":
                self.check_update()
                asndb = pyasn.pyasn(self.asncidr)
                subnets = asndb.get_as_prefixes(asn)
                for s in subnets:
                    print(s)
            elif args.subcommand == "db":
                r = requests.get("https://peeringdb.com/api/net?asn=%i" % asn)
                if r.status_code == 200:
                    if args.json:
                        print(json.dumps(r.json(), sort_keys=False, indent=4))
                    else:
                        data = r.json()["data"][0]
                        print("Name: %s" % data["name"])
                        if data["aka"] != "":
                            print("aka: %s" % data["aka"])
                        if data["notes"] != "":
                            print("Notes: %s" % data["notes"])
                        if data["website"] != "":
                            print("Website: %s" % data["website"])
                        print("Created: %s" % data["created"])
                else:
                    print("ASN not found")
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()



#! /usr/bin/env python
import json
from datetime import datetime

from pyhashlookup.api import Hashlookup, PyHashlookupError

from harpoon.commands.base import Command


class CommandHashLookup(Command):
    """
    # CIRCL Hash Lookup

    **Search a hash in CIRCL Hash lookup base**
    https://circl.lu/services/hashlookup/

    * `harpoon hashlookup hash 8ED4B4ED952526D89899E723F3488DE4`
    """

    name = "hashlookup"
    description = "Request CIRCL Hash Lookup db"
    config = {"HashLookup": []}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("hash", help="Request info on a hash")
        parser_a.add_argument("HASH", help="Hash")
        parser_a.set_defaults(subcommand="hash")
        parser_b = subparsers.add_parser("info", help="Info about the db")
        parser_b.set_defaults(subcommand="info")
        # TODO: bulk query
        self.parser = parser

    def run(self, args, plugins):
        hl = Hashlookup()
        if "subcommand" in args:
            if args.subcommand == "info":
                print(json.dumps(hl.info(), indent=4))
            elif args.subcommand == "hash":
                try:
                    res = hl.lookup(args.HASH)
                except PyHashlookupError:
                    print("Invalid Hash format")
                else:
                    print(json.dumps(res, indent=4))
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_hash(self, query, data):
        print("[+] Checking CIRCL Hash Lookup...")
        hl = Hashlookup()
        try:
            res = hl.lookup(query)
        except PyHashlookupError:
            print("Invalid Hash format")
        else:
            if "FileName" in res:
                # File exist
                data["reports"].append(
                    {
                        "title": "Hash found in CIRCL db: {}".format(res["FileName"]),
                        "source": res["source"],
                        "date": datetime.fromtimestamp(float(res["insert-timestamp"])),
                        "url": "",
                    }
                )



import re
from urllib.parse import quote, urljoin

import requests
from dateutil.parser import parse


class MementoClient(object):
    """
    Implement Memento Protocol
    """
    def __init__(self, base_url='http://archive.is/'):
        self.base_url = base_url
        self.linkre = re.compile(r'^<(?P<url>[^>]+)>; rel="(?P<type>[a-z ]+)"(; datetime="(?P<date>[^"]+)"|,|; type="application/link-format"; from="(?P<from>[^"]+)"; until="(?P<until>[^"]+)")')  # noqa: E501

    def _parselinks(self, data):
        """
        Parse links from RFC 7089, returns list of links
        """
        res = []
        for d in data.split('\n'):
            if d != "":
                regex = self.linkre.match(d)
                if regex is not None:
                    new = {
                        'url': regex.group('url'),
                        'type': regex.group('type')
                    }
                    for i in ['from', 'until', 'date']:
                        if regex.group(i) is not None:
                            if i in regex.groupdict():
                                new[i] = parse(regex.group(i))
                    res.append(new)
        return res

    def snapshots(self, url):
        """
        Download list of snapshots for an url
        """
        r = requests.get(urljoin(self.base_url + 'timemap/', quote(url, safe='')), timeout=3)
        if r.status_code != 200:
            return []
        else:
            links = self._parselinks(r.text)
            # Get original url
            originals = list(
                filter(
                    lambda x: x['type'] == 'original',
                    links
                )
            )
            if len(originals) == 0:
                return []
            original = originals[0]['url']
            # Sort snapshots
            snapshots = []
            for d in links:
                if d['type'] in ['memento', 'first last memento']:
                    snapshots.append({
                        'url': original,
                        'date': d['date'],
                        'archive': d['url']
                    })
            return snapshots



#! /usr/bin/env python
import json

from pycrtsh import Crtsh, CrtshCertificateNotFound

from harpoon.commands.base import Command, Subcommand
from harpoon.lib.utils import unbracket


class SubcommandCert(Subcommand):
    description = "Show details for a certificate"
    cmd = "cert"

    def add_arguments(self, parser):
        parser = parser.add_argument("ID", help="ID, sha1 or sha256 of a certificate")

    def run(self, args):
        crt = Crtsh()
        try:
            if len(args.ID) == 64:
                self.data = crt.get(args.ID, type="sha256")
            elif len(args.ID) == 40:
                self.data = crt.get(args.ID, type="sha1")
            else:
                self.data = crt.get(args.ID, type="id")
        except CrtshCertificateNotFound:
            self.data = None

    def display(self, args):
        if self.data:
            print(json.dumps(self.data, indent=4, default=str))
        else:
            print("Certificate not found")


class SubcommandDomain(Subcommand):
    # TODO: implement in depth list
    description = "Get certificates of a domain"
    cmd = "domain"

    def add_arguments(self, parser):
        parser.add_argument("DOMAIN", help="Domain")
        parser.add_argument(
            "--format",
            "-f",
            choices=["csv", "json", "txt"],
            default="txt",
            help="Output format (default is txt)",
        )

    def run(self, args):
        crt = Crtsh()
        self.data = crt.search(unbracket(args.DOMAIN))

    def display(self, args):
        if args.format == "txt":
            print("Certificates")
            for c in self.data:
                print(
                    "%s\t%s\t%s\t%s"
                    % (
                        c["name"],
                        c["not_before"].isoformat(),
                        c["not_after"].isoformat(),
                        c["id"],
                    )
                )
        elif args.format == "json":
            print(json.dumps(self.data, indent=4, default=str))
        else:
            print("id|name|Issuer|Not Before|Not After")
            for c in self.data:
                print(
                    "{}|{}|{}|{}|{}".format(
                        c["id"],
                        c["name"],
                        ", ".join(
                            [
                                "{}={}".format(a, b)
                                for (a, b) in c["ca"]["parsed_name"].items()
                            ]
                        ),
                        c["not_before"],
                        c["not_after"],
                    )
                )


class SubcommandList(Subcommand):
    description = "Get certificates for a list of domains"
    cmd = "list"

    def add_arguments(self, parser):
        parser.add_argument("FILE", help="File containing a list of domains")
        parser.add_argument(
            "--format",
            "-f",
            choices=["csv", "json", "txt"],
            default="txt",
            help="Output format (default is txt)",
        )

    def run(self, args):
        crt = Crtsh()

        # Get the list of domains
        with open(args.FILE, "r") as f:
            domains = [a.strip() for a in f.read().split()]

        # Get certificates
        self.data = []
        for d in domains:
            self.data.extend(crt.search(d))

    def display(self, args):
        if args.format == "txt":
            print("Certificates")
            for c in self.data:
                print(
                    "%s\t%s\t%s\t%s"
                    % (
                        c["name"],
                        c["not_before"].isoformat(),
                        c["not_after"].isoformat(),
                        c["id"],
                    )
                )
        elif args.format == "json":
            print(json.dumps(self.data, indent=4, default=str))
        else:
            print("id|name|Issuer|Not Before|Not After")
            for c in self.data:
                print(
                    "{}|{}|{}|{}|{}".format(
                        c["id"],
                        c["name"],
                        ", ".join(
                            [
                                "{}={}".format(a, b)
                                for (a, b) in c["ca"]["parsed_name"].items()
                            ]
                        ),
                        c["not_before"],
                        c["not_after"],
                    )
                )


class SubcommandSubdomains(Subcommand):
    description = "Search subdomains of a domain through certificates"
    cmd = "subdomains"

    def add_arguments(self, parser):
        parser.add_argument("DOMAIN", help="Domain")

    def run(self, args):
        crt = Crtsh()
        subdomains = set()
        index = crt.search(unbracket(args.DOMAIN))
        for c in index:
            data = crt.get(c["id"], type="id")
            subdomains.add(data["subject"]["commonName"])
            if "alternative_names" in data["extensions"]:
                for d in data["extensions"]["alternative_names"]:
                    subdomains.add(d)
        self.data = list(subdomains)

    def display(self, args):
        for d in self.data:
            print(d)


class CommandCertsh(Command):
    """
    # Crt.sh

    **Search and download information from https://crt.sh/

    * Search all certificates from a domain: `harpoon crtsh domain amnesty.org`
    * Search all certificates from a list of domains in a file with CSV output: `harpoon crtsh list FILE -f CSV`
    * Search for subdomains of a domain : `harpoon crtsh subdomains DOMAIN`
    * Show details of a certificate : `harpoon crtsh cert SHA1`
    """

    name = "crtsh"
    description = "Search in https://crt.sh/ (Certificate Transparency database)"
    config = {"Crtsh": []}

    def __init__(self, config):
        super().__init__(config=config)
        self.add_subcommand(SubcommandCert)
        self.add_subcommand(SubcommandDomain)
        self.add_subcommand(SubcommandList)
        self.add_subcommand(SubcommandSubdomains)

    def intel_domain(self, query, data):
        print("[+] Checking Crtsh...")
        crt = Crtsh()
        subdomains = set()
        index = crt.search(unbracket(query))
        for c in index:
            data = crt.get(c["id"], type="id")
            subdomains.add(data["subject"]["commonName"])
            if "alternative_names" in data["extensions"]:
                for d in data["extensions"]["alternative_names"]:
                    data["subdomains"].add(d)



from urllib import parse

import requests


class OpenCageError(Exception):
    pass


class RateLimitExceededError(OpenCageError):
    pass


class OpenCage:
    def __init__(self, key):
        self.url = 'http://api.opencagedata.com/geocode/v1/json'
        self.key = key

    def _request(self, params):
        params["key"] = self.key
        r = requests.get(
            self.url,
            params=params
        )
        if r.status_code != 200:
            if (r.status_code == 402 or r.status_code == 429):
                raise RateLimitExceededError()
            else:
                raise OpenCageError()
        return r.json()

    def reverse(self, lat, long):
        """
        Find the address closest to a location
        """
        params = {
            "q": "{}%2C%20{}".format(lat, long),
            "language": "en"
        }
        return self._request(params)

    def geocode(self, place):
        params = {
            "q": parse.quote(place),
            "language": "en"
        }
        return self._request(params)



#! /usr/bin/env python
import sys

from github import Github, UnknownObjectException
from github.GithubException import RateLimitExceededException

from harpoon.commands.base import Command


class CommandGithub(Command):
    """
    # Github

    **Request Github API**

    * Get information on a repository: `harpoon github repo kneufeld/consolemd`
    * Search for information in Github code: `harpoon github search randhome.io`
    * Search for information in Github repo: `harpoon github search -t repo harpoon`
    """

    name = "github"
    description = "Request Github information through the API"
    config = {"Github": ["token"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="SubCommands")
        parser_a = subparsers.add_parser("search", help="Search in github")
        parser_a.add_argument(
            "--type",
            "-t",
            choices=["repo", "code", "commit", "issues", "wikis", "users"],
            default="code",
            help="Type of data to search",
        )
        parser_a.add_argument(
            "--limit", "-l", default="10", type=int, help="Result limit"
        )
        parser_a.add_argument("SEARCH")
        parser_a.set_defaults(subcommand="search")
        parser_b = subparsers.add_parser(
            "repo", help="Information on a github repository"
        )
        parser_b.add_argument("REPOSITORY")
        parser_b.add_argument(
            "-o",
            "--only-emails",
            action="store_true",
            help="Only list emails of committers",
        )
        parser_b.set_defaults(subcommand="repo")

        self.parser = parser

    def run(self, args, plugins):
        g = Github(self._config_data["Github"]["token"])
        if "subcommand" in args:
            if args.subcommand == "search":
                if args.type == "code":
                    res = g.search_code(args.SEARCH)
                    nb = 0
                    for i in res:
                        print("[+] %s" % i.html_url)
                        print(i.decoded_content[:300])
                        print("")
                        nb += 1
                        if nb > args.limit:
                            sys.exit(0)
                elif args.type == "repo":
                    res = g.search_repositories(args.SEARCH)
                    nb = 0
                    for i in res:
                        print("[+] %s by %s" % (i.name, i.owner.name))
                        print("\t%s" % i.description)
                        print("\t%s" % i.html_url)
                        print("\t%s" % i.language)
                        print(
                            "\t[Watch: %i][Stars: %i][Forks: %i]"
                            % (i.watchers, i.stargazers_count, i.forks_count)
                        )
                        print("")
                        nb += 1
                        if nb > args.limit:
                            sys.exit(0)
                elif args.type == "commit":
                    # Not yet implemented by PyGithub
                    raise Exception("Not yet implemented")
            elif args.subcommand == "repo":
                # clean input
                if args.REPOSITORY.startswith("https://github.com"):
                    rep_name = args.REPOSITORY[19:]
                else:
                    rep_name = args.REPOSITORY
                if rep_name.endswith(".git"):
                    rep_name = rep_name[:-4]
                repo = g.get_repo(rep_name)
                # Check if found
                try:
                    repo.id
                except UnknownObjectException:
                    print("Repository not found")
                    return
                if args.only_emails:
                    # FIXME: really slow
                    committers = set()
                    for c in repo.get_commits():
                        if c.committer:
                            if c.committer.email:
                                committers.add(c.committer.email)
                    for c in committers:
                        print(c)
                else:
                    print("-Name: %s" % repo.full_name)
                    print("-Owner: %s %s" % (repo.owner.login, repo.owner.email))
                    print("-Language: %s" % repo.language)
                    print(
                        "-%i Watchers / %i Stars / %i Forks"
                        % (repo.watchers, repo.stargazers_count, repo.forks_count)
                    )
                    # FIXME: really slow
                    committers = {}
                    for c in repo.get_commits():
                        if c.committer:
                            if c.committer.email:
                                if c.committer.email in committers:
                                    committers[c.committer.email] += 1
                                else:
                                    committers[c.committer.email] = 1
                    print("-Committers:")
                    for committer in sorted(committers.items()):
                        print("\t%s %i" % (committer[0], committer[1]))

            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_email(self, query, data):
        """
        Check for emails in Github pages
        """
        print("[+] Checking Github...")
        g = Github(self._config_data["Github"]["token"])

        try:
            res = g.search_code(query)
        except RateLimitExceededException:
            print("Rate limit exceeded on Github")
            return

        for i in res:
            data["mentions"].append({"url": i.html_url, "source": "Github"})



#! /usr/bin/env python
import json

import pytz
from dateutil.parser import parse
from pybinaryedge import BinaryEdge, BinaryEdgeException, BinaryEdgeNotFound

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandBinaryEdge(Command):
    """
    # binaryedge plugin

    **Query the BinaryEdge API https://www.binaryedge.io/**

    * Check details on an IP: `harpoon binaryedge ip IP`
    * Search for a specific query: `harpoon binaryedge search QUERY`
    """

    name = "binaryedge"
    description = "Request BinaryEdge API"
    config = {"BinaryEdge": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Commands")
        parser_b = subparsers.add_parser("ip", help="Query an IP address")
        parser_b.add_argument("IP", help="IP to be requested")
        parser_b.add_argument(
            "--historical",
            "-H",
            action="store_true",
            help="Requests historical data about an IP",
        )
        parser_b.add_argument(
            "--score",
            "-s",
            action="store_true",
            help="Requests vulnerability score about an IP",
        )
        parser_b.add_argument(
            "--image",
            "-i",
            action="store_true",
            help="Requests images identified for an IP",
        )
        parser_b.add_argument(
            "--torrent",
            "-t",
            action="store_true",
            help="Request torrents identified for an IP",
        )
        parser_b.add_argument(
            "--dns",
            "-d",
            action="store_true",
            help="Requests images identified for an IP",
        )
        parser_b.add_argument(
            "--page", "-p", type=int, default=1, help="Get specific page"
        )
        parser_b.set_defaults(which="ip")
        parser_c = subparsers.add_parser("search", help="Search in the database")
        parser_c.add_argument("SEARCH", help="Search request")
        parser_c.add_argument(
            "--page", "-p", type=int, default=1, help="Get specific page"
        )
        parser_c.add_argument(
            "--image",
            "-i",
            action="store_true",
            help="Requests images identified for an IP",
        )
        parser_c.set_defaults(which="search")
        parser_d = subparsers.add_parser(
            "dataleaks", help="Search in the leaks database"
        )
        parser_d.add_argument("EMAIL", help="Search email in the leaks database")
        parser_d.add_argument(
            "--domain",
            "-d",
            action="store_true",
            help="Search for domain instead of email",
        )
        parser_d.set_defaults(which="dataleaks")
        parser_e = subparsers.add_parser(
            "domain", help="Search information on a domain"
        )
        parser_e.add_argument("DOMAIN", help="Domain to be requested")
        parser_e.add_argument(
            "--page", "-p", type=int, default=1, help="Get specific page"
        )
        parser_e.add_argument(
            "--subdomains", "-s", action="store_true", help="Returns subdomains"
        )
        parser_e.set_defaults(which="domain")
        self.parser = parser

    def run(self, args, plugins):
        be = BinaryEdge(self._config_data["BinaryEdge"]["key"])
        try:
            if hasattr(args, "which"):
                if args.which == "ip":
                    if args.score:
                        res = be.host_score(unbracket(args.IP))
                    elif args.image:
                        res = be.image_ip(unbracket(args.IP))
                    elif args.torrent:
                        if args.historical:
                            res = be.torrent_historical_ip(unbracket(args.IP))
                        else:
                            res = be.torrent_ip(unbracket(args.IP))
                    elif args.historical:
                        res = be.host_historical(unbracket(args.IP))
                    elif args.dns:
                        res = be.domain_ip(args.IP, page=args.page)
                    else:
                        res = be.host(unbracket(args.IP))
                    print(json.dumps(res, sort_keys=True, indent=4))
                elif args.which == "search":
                    if args.image:
                        res = be.image_search(args.SEARCH, page=args.page)
                    else:
                        res = be.host_search(args.SEARCH, page=args.page)
                    print(json.dumps(res, sort_keys=True, indent=4))
                elif args.which == "dataleaks":
                    if args.domain:
                        res = be.dataleaks_organization(args.EMAIL)
                    else:
                        res = be.dataleaks_email(args.EMAIL)
                    print(json.dumps(res, sort_keys=True, indent=4))
                elif args.which == "domain":
                    if args.subdomains:
                        res = be.domain_subdomains(args.DOMAIN, page=args.page)
                    else:
                        res = be.domain_dns(args.DOMAIN, page=args.page)
                    print(json.dumps(res, sort_keys=True, indent=4))
                else:
                    self.parser.print_help()
            else:
                self.parser.print_help()
        except ValueError as e:
            print("Invalid Value: %s" % e.message)
        except BinaryEdgeNotFound:
            print("Search term not found")
        except BinaryEdgeException as e:
            print("Error: %s" % e.message)

    def intel_domain(self, query, data):
        print("[+] Downloading BinaryEdge information....")
        try:
            be = BinaryEdge(self._config_data["BinaryEdge"]["key"])
            res = be.domain_dns(query)
            for d in res["events"]:
                if "A" in d:
                    for a in d["A"]:
                        data["passive_dns"].append(
                            {
                                "ip": a,
                                "first": parse(d["updated_at"]).astimezone(pytz.utc),
                                "last": parse(d["updated_at"]).astimezone(pytz.utc),
                                "source": "BinaryEdge",
                            }
                        )
        except BinaryEdgeException:
            print("You need a paid BinaryEdge subscription for this request")

    def intel_ip(self, query, data):
        print("[+] Downloading BinaryEdge information....")
        try:
            be = BinaryEdge(self._config_data["BinaryEdge"]["key"])
            res = be.domain_ip(query)
            for d in res["events"]:
                data["passive_dns"].append(
                    {
                        "domain": d["domain"],
                        "first": parse(d["updated_at"]).astimezone(pytz.utc),
                        "last": "",
                        "source": "BinaryEdge",
                    }
                )
            res = be.host(query)
            for d in res["events"]:
                data["ports"].append(
                    {"port": d["port"], "info": "", "source": "BinaryEdge"}
                )
        except BinaryEdgeException:
            print("You need a paid BinaryEdge subscription for this request")



#! /usr/bin/env python
from harpoon.commands.base import Command
from harpoon.lib.pgp import Pgp


class CommandPgp(Command):
    """
    # PGP

    **Searches for PGP keys in MIT key servers**

    Example:
    ```
    harpoon pgp search lemonde.fr
    [+] 0x07A0514E0F568618	2017-12-19	Anne Michel anne.michel@lemonde.fr
    [+] 0xAE95106F1B5A0D7E	2017-11-28	perso maxime.loliee@gmail.com
            loliee maxime.loliee@gmail.com
            pro maxime@siliadev.com
            lemonde loliee@lemonde.fr
    [SNIP]
    ```

    Option `-o` just print the list of emails (no information on keys)
    """

    name = "pgp"
    description = "Search for information in PGP key servers"
    config = {"PGP": []}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommands")
        parser_a = subparsers.add_parser("search", help="Search in PGP server")
        parser_a.add_argument("SEARCH", help="Query")
        parser_a.add_argument(
            "--only-emails",
            "-o",
            action="store_true",
            help="Print only email addresses",
        )
        parser_a.set_defaults(subcommand="search")
        self.parser = parser

    def run(self, args, plugins):
        if hasattr(args, "subcommand"):
            if args.subcommand == "search":
                res = Pgp.search(args.SEARCH)
                if len(res):
                    if args.only_emails:
                        emails = set()
                        for r in res:
                            for e in r["emails"]:
                                if args.SEARCH.lower() in e[1].lower():
                                    emails.add(e[1])
                        for e in emails:
                            print(e)
                    else:
                        for r in res:
                            print(
                                "[+] %s\t%s\t%s %s"
                                % (
                                    r["id"],
                                    r["date"].strftime("%Y-%m-%d"),
                                    r["emails"][0][0],
                                    r["emails"][0][1],
                                )
                            )
                            if len(r["emails"]) > 1:
                                for e in r["emails"][1:]:
                                    print("\t\t\t\t\t%s %s" % (e[0], e[1]))
                else:
                    print("No results (could be too many results)")
                    print(
                        "Double check at http://pgp.mit.edu/pks/lookup?search=%s"
                        % args.SEARCH
                    )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel_email(self, query, data):
        print("[+] Checking PGP servers...")
        res = Pgp.search(query)
        for r in res:
            data["keys"].append(
                {
                    "date": r["date"],
                    "id": r["id"],
                    "name": r["emails"][0][0],
                    "email": r["emails"][0][1],
                }
            )



#! /usr/bin/env python
from datetime import datetime

import pytz

from harpoon.commands.base import Command
from harpoon.lib.dnsdb import DnsDB, DNSDBError
from harpoon.lib.utils import ts_to_str, unbracket


class DnsDbTotal(Command):
    """
    # Farsight DnsDB

    * On a domain : `harpoon dnsdb domain DOMAIN`
    * On an IP : `harpoon dnsdb ip IP`
    """

    name = "dnsdb"
    description = "Requests Farsight DNSDB"
    config = {"Dnsdb": ["key"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser(
            "ip", help="Request passive DNS info on an IP address"
        )
        parser_a.add_argument("IP", help="IP to be queried")
        parser_a.set_defaults(subcommand="ip")
        parser_b = subparsers.add_parser(
            "domain", help="Request passive DNS info on a domain"
        )
        parser_b.add_argument("DOMAIN", help="DOMAIN to be queried")
        parser_b.set_defaults(subcommand="domain")
        self.parser = parser

    def run(self, args, plugins):
        dnsdb = DnsDB(self._config_data["Dnsdb"]["key"])
        if "subcommand" in args:
            if args.subcommand == "domain":
                try:
                    results = dnsdb.rrset_lookup(unbracket(args.DOMAIN))
                except DNSDBError as e:
                    print("Request failed : {}".format(e))
                else:
                    for r in results:
                        print(
                            "{}\t{}\t{}\t{}".format(
                                r["rrtype"],
                                ts_to_str(r["time_first"]),
                                ts_to_str(r["time_last"]),
                                "/ ".join(r["rdata"]),
                            )
                        )
            elif args.subcommand == "ip":
                try:
                    results = dnsdb.rdata_lookup(unbracket(args.IP), type="ip")
                except DNSDBError as e:
                    print("Request failed : {}".format(e))
                else:
                    if len(results) == 0:
                        print("No resuts for this IP")
                    for r in results:
                        print(
                            "{}\t{}\t{}\t{}".format(
                                r["rrtype"],
                                ts_to_str(r["time_first"]),
                                ts_to_str(r["time_last"]),
                                r["rrname"],
                            )
                        )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "domain":
            print("[+] Checking DNSdb...")
            dnsdb = DnsDB(self._config_data["Dnsdb"]["key"])
            try:
                results = dnsdb.rrset_lookup(unbracket(query))
            except DNSDBError:
                print("Request failed")
                return
            for r in results:
                if r["rrtype"] in ["A", "AAAA"]:
                    for ip in r["rdata"]:
                        data["passive_dns"].append(
                            {
                                "ip": ip.strip(),
                                "first": datetime.fromtimestamp(
                                    r["time_first"]
                                ).astimezone(pytz.utc),
                                "last": datetime.fromtimestamp(
                                    r["time_last"]
                                ).astimezone(pytz.utc),
                                "source": "DNSdb",
                            }
                        )
        elif type == "ip":
            print("[+] Checking DNSdb...")
            dnsdb = DnsDB(self._config_data["Dnsdb"]["key"])
            try:
                results = dnsdb.rdata_lookup(unbracket(query), type="ip")
            except DNSDBError:
                print("Request to DNSDB failed")
                return
            for r in results:
                if r["rrtype"] in ["A", "AAAA"]:
                    data["passive_dns"].append(
                        {
                            "domain": r["rrname"].strip(),
                            "first": datetime.fromtimestamp(r["time_first"]).astimezone(
                                pytz.utc
                            ),
                            "last": datetime.fromtimestamp(r["time_last"]).astimezone(
                                pytz.utc
                            ),
                            "source": "DNSdb",
                        }
                    )



#! /usr/bin/env python
import hashlib
import json
import os
import sys
import time

import pytz
from dateutil.parser import parse
from virus_total_apis import PrivateApi, PublicApi

from harpoon.commands.base import Command
from harpoon.lib.utils import unbracket


class CommandVirusTotal(Command):
    """
    # Virus Total

    * Search for hash: `harpoon vt hash HASH`
    * Search for a domain: `harpoon vt domain example.org`
    * Search for an IP: `harpoon vt ip IP`
    * Check a file in VT: `harpoon vt file FILE` (check for the hash, no upload)
    * Download a file (private only): `harpoon vt dl HASH`
    * Search for a list of hashes from a file: `harpoon vt hashlist FILE`
    * Search for a list of domains from a file : `harpoon vt domainlist FILE`
    * Search for a list of IP addresses from a file : `harpoon vt iplist FILE`
    """

    name = "vt"
    description = "Request Virus Total API"
    config = {"VirusTotal": ["key", "type"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("hash", help="Request info on a hash")
        parser_a.add_argument("HASH", help="Hash")
        parser_a.add_argument("--raw", "-r", help="Raw data", action="store_true")
        parser_a.add_argument(
            "--extended",
            "-e",
            help="Extended info (private API only)",
            action="store_true",
        )
        parser_a.set_defaults(subcommand="hash")
        parser_b = subparsers.add_parser("hashlist", help="Request a list of hashes")
        parser_b.add_argument("FILE", help="File containing the domains")
        parser_b.set_defaults(subcommand="hashlist")
        parser_c = subparsers.add_parser("domain", help="Request info on a domain")
        parser_c.add_argument("DOMAIN", help="Domain")
        parser_c.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_c.set_defaults(subcommand="domain")
        parser_d = subparsers.add_parser("ip", help="Request info on an IP")
        parser_d.add_argument("IP", help="IP")
        parser_d.set_defaults(subcommand="ip")
        parser_e = subparsers.add_parser("url", help="Request info on an URL")
        parser_e.add_argument("URL", help="URL")
        parser_e.set_defaults(subcommand="url")
        parser_f = subparsers.add_parser(
            "domainlist", help="Request info on a list of domains"
        )
        parser_f.add_argument("FILE", help="File containing the list of domains")
        parser_f.set_defaults(subcommand="domainlist")
        parser_e = subparsers.add_parser(
            "file", help="Request info on a file (no upload)"
        )
        parser_e.add_argument("FILE", help="File")
        parser_e.add_argument("--raw", "-r", help="Raw data", action="store_true")
        parser_e.set_defaults(subcommand="file")
        parser_f = subparsers.add_parser("dl", help="Download a file from VT")
        parser_f.add_argument("HASH", help="Hash of the file")
        parser_f.set_defaults(subcommand="dl")
        parser_g = subparsers.add_parser(
            "iplist", help="Request info on a list of IP addresses"
        )
        parser_g.add_argument("FILE", help="File containing the list of IPs")
        parser_g.set_defaults(subcommand="iplist")
        self.parser = parser

    def print_domaininfo(self, res):
        """Print nicely the domain information"""
        if "results" in res:
            if "verbose_msg" in res["results"]:
                print(res["results"]["verbose_msg"])
            if "detected_urls" in res["results"]:
                if len(res["results"]["detected_urls"]) > 0:
                    print("-Detected urls:")
                    for r in res["results"]["detected_urls"]:
                        print(
                            "\t%s (on %s, %i/%i)"
                            % (r["url"], r["scan_date"], r["positives"], r["total"])
                        )
            if "undetected_urls" in res["results"]:
                print("-Undetected urls:")
                if len(res["results"]["undetected_urls"]) > 0:
                    for r in res["results"]["undetected_urls"]:
                        print("\t%s (on %s, %i/%i)" % (r[0], r[4], r[2], r[3]))
            if "resolutions" in res["results"]:
                if len(res["results"]["resolutions"]) > 0:
                    print("-Resolutions:")
                    for r in res["results"]["resolutions"]:
                        print("\t%s (%s)" % (r["ip_address"], r["last_resolved"]))
            if "detected_referrer_samples" in res["results"]:
                if len(res["results"]["detected_referrer_samples"]) > 0:
                    print("-Detected Referrer Sample:")
                    for r in res["results"]["detected_referrer_samples"]:
                        print(
                            "\t%s (%i/%i)" % (r["sha256"], r["positives"], r["total"])
                        )
            if "undetected_referrer_samples" in res["results"]:
                if len(res["results"]["undetected_referrer_samples"]) > 0:
                    print("-Undetected Referrer Sample:")
                    for r in res["results"]["undetected_referrer_samples"]:
                        print(
                            "\t%s (%i/%i)" % (r["sha256"], r["positives"], r["total"])
                        )
            if "undetected_downloaded_samples" in res["results"]:
                if len(res["results"]["undetected_downloaded_samples"]) > 0:
                    print("-Undetected Downloaded Sample:")
                    for r in res["results"]["undetected_downloaded_samples"]:
                        print(
                            "\t%s (on %s, %i/%i)"
                            % (r["sha256"], r["date"], r["positives"], r["total"])
                        )
            if "detected_downloaded_samples" in res["results"]:
                if len(res["results"]["detected_downloaded_samples"]) > 0:
                    print("-Detected Downloaded Sample:")
                    for r in res["results"]["detected_downloaded_samples"]:
                        print(
                            "\t%s (on %s, %i/%i)"
                            % (r["sha256"], r["date"], r["positives"], r["total"])
                        )

    def print_file(self, response):
        """
        Print details on a file
        """
        if response["response_code"] != 200:
            print(
                "Error with the request (reponse code %i)" % response["response_code"]
            )
            sys.exit(1)
        if response["results"]["response_code"] == 0:
            print("File not found")
            sys.exit(0)
        print(
            "[+] Detection: %i / %i"
            % (response["results"]["positives"], response["results"]["total"])
        )
        print("[+] MD5: %s" % response["results"]["md5"])
        print("[+] SHA1: %s" % response["results"]["sha1"])
        print("[+] SHA256: %s" % response["results"]["sha256"])
        if "first_seen" in response["results"]:
            print("[+] First Seen: %s" % response["results"]["first_seen"])
        if "last_seen" in response["results"]:
            print("[+] Last Seen: %s" % response["results"]["last_seen"])
        print("[+] Link: %s" % response["results"]["permalink"])

    def run(self, args, plugins):
        if "subcommand" in args:
            if self._config_data["VirusTotal"]["type"] != "public":
                vt = PrivateApi(self._config_data["VirusTotal"]["key"])
                if args.subcommand == "hash":
                    response = vt.get_file_report(args.HASH)
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                        if args.extended:
                            response = vt.get_network_traffic(args.HASH)
                            print(json.dumps(response, sort_keys=False, indent=4))
                            response = vt.get_file_behaviour(args.HASH)
                            print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "dl":
                    if os.path.isfile(args.HASH):
                        print("File %s already exists" % args.HASH)
                        sys.exit(0)
                    data = vt.get_file(args.HASH)
                    if isinstance(data, dict):
                        if "results" in data:
                            with open(args.HASH, "wb") as f:
                                f.write(data["results"])
                            print("File downloaded as %s" % args.HASH)
                        else:
                            print("Invalid answer format")
                            sys.exit(1)
                    else:
                        with open(args.HASH, "wb") as f:
                            f.write(data)
                        print("File downloaded as %s" % args.HASH)

                elif args.subcommand == "file":
                    with open(args.FILE, "rb") as f:
                        # FIXME : could be more efficient
                        data = f.read()
                    m = hashlib.sha256()
                    m.update(data)
                    h = m.hexdigest()
                    response = vt.get_file_report(h)
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "hashlist":
                    with open(args.FILE, "r") as infile:
                        data = infile.read().split()
                    hash_list = list(set([a.strip() for a in data]))
                    print("Hash;Found;Detection;Total AV;First Seen;Last Seen;Link")
                    for h in hash_list:
                        response = vt.get_file_report(h)
                        if response["response_code"] != 200:
                            print(
                                "Error with the request (reponse code %i)"
                                % response["response_code"]
                            )
                            print(json.dumps(response, sort_keys=False, indent=4))
                            print("Quitting...")
                            sys.exit(1)
                        if "response_code" in response["results"]:
                            if response["results"]["response_code"] == 0:
                                print("%s;Not found;;;;;" % h)
                            else:
                                print(
                                    "{};Found;{};{};{};{}".format(
                                        h,
                                        response["results"]["positives"],
                                        response["results"]["total"],
                                        response["results"]["scan_date"],
                                        response["results"]["permalink"],
                                    )
                                )
                        else:
                            print("%s;Not found;;;;;" % h)
                        #  Max 4 requests per minute
                        time.sleep(15)
                elif args.subcommand == "domainlist":
                    with open(args.FILE, "r") as infile:
                        data = infile.read().split()
                    for d in data:
                        print("################ Domain %s" % d.strip())
                        res = vt.get_domain_report(d.strip())
                        self.print_domaininfo(res)
                elif args.subcommand == "iplist":
                    with open(args.FILE, "r") as infile:
                        data = infile.read().split()
                    for d in data:
                        print("################ IP %s" % d.strip())
                        res = vt.get_ip_report(unbracket(d.strip()))
                        print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "domain":
                    res = vt.get_domain_report(unbracket(args.DOMAIN))
                    if args.json:
                        print(json.dumps(res, sort_keys=False, indent=4))
                    else:
                        self.print_domaininfo(res)
                elif args.subcommand == "ip":
                    res = vt.get_ip_report(unbracket(args.IP))
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "url":
                    res = vt.get_url_report(args.URL)
                    print(json.dumps(res, sort_keys=False, indent=4))
                else:
                    self.parser.print_help()
            else:
                vt = PublicApi(self._config_data["VirusTotal"]["key"])
                if args.subcommand == "hash":
                    response = vt.get_file_report(args.HASH)
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "file":
                    with open(args.FILE, "rb") as f:
                        # FIXME : could be more efficient
                        data = f.read()
                    m = hashlib.sha256()
                    m.update(data)
                    response = vt.get_file_report(m.hexdigest())
                    if args.raw:
                        print(json.dumps(response, sort_keys=False, indent=4))
                    else:
                        self.print_file(response)
                elif args.subcommand == "hashlist":
                    with open(args.FILE, "r") as infile:
                        data = infile.read().split()
                    hash_list = list(set([a.strip() for a in data]))
                    print("Hash;Found;Detection;Total AV;Link")
                    for h in hash_list:
                        response = vt.get_file_report(h)
                        if response["response_code"] != 200:
                            print(
                                "Error with the request (reponse code %i)"
                                % response["response_code"]
                            )
                            print(json.dumps(response, sort_keys=False, indent=4))
                            print("Quitting...")
                            sys.exit(1)
                        if "response_code" in response["results"]:
                            if response["results"]["response_code"] == 0:
                                print("%s;Not found;;;" % h)
                            else:
                                print(
                                    "%s;Found;%i;%i;%s"
                                    % (
                                        h,
                                        response["results"]["positives"],
                                        response["results"]["total"],
                                        response["results"]["permalink"],
                                    )
                                )
                        else:
                            print("%s;Not found;;;" % h)
                elif args.subcommand == "domain":
                    res = vt.get_domain_report(unbracket(args.DOMAIN))
                    if args.json:
                        print(json.dumps(res, sort_keys=False, indent=4))
                    else:
                        self.print_domaininfo(res)
                elif args.subcommand == "ip":
                    res = vt.get_ip_report(unbracket(args.IP))
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "url":
                    res = vt.get_url_report(args.URL)
                    print(json.dumps(res, sort_keys=False, indent=4))
                elif args.subcommand == "domainlist":
                    print(
                        "Not implemented yet with public access, please propose PR if you need it"
                    )
                elif args.subcommand == "dl":
                    print(
                        "VirusTotal does not allow downloading files with a public feed, sorry"
                    )
                    sys.exit(0)
                else:
                    self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "domain":
            if self._config_data["VirusTotal"]["type"] != "public":
                print("[+] Checking VirusTotal....")
                vt = PrivateApi(self._config_data["VirusTotal"]["key"])
                res = vt.get_domain_report(query)
                if "results" in res:
                    if "resolutions" in res["results"]:
                        for r in res["results"]["resolutions"]:
                            try:
                                data["passive_dns"].append(
                                    {
                                        "first": parse(r["last_resolved"]).astimezone(
                                            pytz.utc
                                        ),
                                        "last": parse(r["last_resolved"]).astimezone(
                                            pytz.utc
                                        ),
                                        "ip": r["ip_address"],
                                        "source": "VT",
                                    }
                                )
                            except TypeError:
                                # Error with the date
                                pass
                    if "undetected_downloaded_samples" in res["results"]:
                        for r in res["results"]["undetected_downloaded_samples"]:
                            data["files"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc)
                                    if "date" in r
                                    else "",
                                    "source": "VT",
                                }
                            )
                    if "undetected_referrer_samples" in res["results"]:
                        for r in res["results"]["undetected_referrer_samples"]:
                            data["files"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc)
                                    if "date" in r
                                    else "",
                                    "source": "VT",
                                }
                            )
                    if "undetected_communicating_samples" in res["results"]:
                        for r in res["results"]["undetected_communicating_samples"]:
                            data["malware"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc),
                                    "source": "VT",
                                }
                            )
                    if "detected_communicating_samples" in res["results"]:
                        for r in res["results"]["detected_communicating_samples"]:
                            data["malware"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc),
                                    "source": "VT",
                                }
                            )
                    if "detected_downloaded_samples" in res["results"]:
                        for r in res["results"]["detected_downloaded_samples"]:
                            data["malware"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc),
                                    "source": "VT",
                                }
                            )
                    if "detected_referrer_samples" in res["results"]:
                        for r in res["results"]["detected_referrer_samples"]:
                            if "date" in r:
                                data["malware"].append(
                                    {
                                        "hash": r["sha256"],
                                        "date": parse(r["date"]).astimezone(pytz.utc),
                                        "source": "VT",
                                    }
                                )
                    if "detected_urls" in res["results"]:
                        for r in res["results"]["detected_urls"]:
                            data["urls"].append(
                                {
                                    "date": parse(r["scan_date"]).astimezone(pytz.utc),
                                    "url": r["url"],
                                    "ip": "",
                                    "source": "VT",
                                }
                            )
        elif type == "ip":
            if self._config_data["VirusTotal"]["type"] != "public":
                print("[+] Checking VirusTotal...")
                vt = PrivateApi(self._config_data["VirusTotal"]["key"])
                res = vt.get_ip_report(query)
                if "results" in res:
                    if "resolutions" in res["results"]:
                        for r in res["results"]["resolutions"]:
                            try:
                                data["passive_dns"].append(
                                    {
                                        "first": parse(r["last_resolved"]).astimezone(
                                            pytz.utc
                                        ),
                                        "last": parse(r["last_resolved"]).astimezone(
                                            pytz.utc
                                        ),
                                        "domain": r["hostname"],
                                        "source": "VT",
                                    }
                                )
                            except TypeError:
                                # Error with the date
                                pass
                    if "undetected_downloaded_samples" in res["results"]:
                        for r in res["results"]["undetected_downloaded_samples"]:
                            data["files"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc)
                                    if "date" in r
                                    else "",
                                    "source": "VT",
                                }
                            )
                    if "undetected_referrer_samples" in res["results"]:
                        for r in res["results"]["undetected_referrer_samples"]:
                            data["files"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc)
                                    if "date" in r
                                    else "",
                                    "source": "VT",
                                }
                            )
                    if "undetected_communicating_samples" in res["results"]:
                        for r in res["results"]["undetected_communicating_samples"]:
                            data["malware"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc),
                                    "source": "VT",
                                }
                            )
                    if "detected_communicating_samples" in res["results"]:
                        for r in res["results"]["detected_communicating_samples"]:
                            data["malware"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc),
                                    "source": "VT",
                                }
                            )
                    if "detected_downloaded_samples" in res["results"]:
                        for r in res["results"]["detected_downloaded_samples"]:
                            data["malware"].append(
                                {
                                    "hash": r["sha256"],
                                    "date": parse(r["date"]).astimezone(pytz.utc),
                                    "source": "VT",
                                }
                            )
                    if "detected_urls" in res["results"]:
                        for r in res["results"]["detected_urls"]:
                            data["urls"].append(
                                {
                                    "date": parse(r["scan_date"]).astimezone(pytz.utc),
                                    "url": r["url"],
                                    "ip": "",
                                    "source": "VT",
                                }
                            )
        elif type == "hash":
            if self._config_data["VirusTotal"]["type"] != "public":
                print("[+] Checking VirusTotal...")
                vt = PrivateApi(self._config_data["VirusTotal"]["key"])
                res = vt.get_file_report(query)
                if res["results"]["response_code"] == 1:
                    # Found
                    data["samples"].append(
                        {
                            "date": parse(res["results"]["scan_date"]).astimezone(
                                pytz.utc
                            ),
                            "source": "VT",
                            "url": res["results"]["permalink"],
                            "infos": {
                                "AV Result": "{} / {}".format(
                                    res["results"]["positives"], res["results"]["total"]
                                ),
                                "First Seen": res["results"]["first_seen"],
                                "File Names": ", ".join(
                                    res["results"]["submission_names"][:5]
                                ),
                            },
                        }
                    )
                    if "ITW_urls" in res["results"]:
                        for url in res["results"]["ITW_urls"]:
                            data["urls"].append(
                                {
                                    "url": url,
                                    "source": "VT",
                                    "link": res["results"]["permalink"],
                                }
                            )
                    if "additional_info" in res["results"]:
                        if "behaviour-v1" in res["results"]["additional_info"]:
                            if (
                                "network"
                                in res["results"]["additional_info"]["behaviour-v1"]
                            ):
                                for d in res["results"]["additional_info"][
                                    "behaviour-v1"
                                ]["network"]["dns"]:
                                    data["network"].append(
                                        {
                                            "source": "VT",
                                            "url": res["results"]["permalink"],
                                            "host": d["hostname"],
                                            "ip": d["ip"],
                                        }
                                    )



#! /usr/bin/env python
import json

import pytz

from harpoon.commands.base import Command
from harpoon.lib.robtex import Robtex, RobtexError
from harpoon.lib.utils import json_serial, unbracket


class CommandRobtex(Command):
    """
    # Robtex

    **Queries Robtex API https://www.robtex.com/api/**

    * Get information on an IP address: `harpoon robtex ip IP`
    * Get information on an ASN: `harpoon robtex asn 18342`
    * get information on a domain: `harpoon robtex domain example.org`

    The option `-j` shows raw JSON output.
    """

    name = "robtex"
    description = "Search in Robtex API (https://www.robtex.com/api/)"
    config = {"RobTex": []}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("ip", help="Request info on an IP")
        parser_a.add_argument("IP", help="IP")
        parser_a.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_a.set_defaults(subcommand="ip")
        parser_b = subparsers.add_parser("asn", help="Request info on an ASN")
        parser_b.add_argument("ASN", help="ASN", type=int)
        parser_b.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_b.set_defaults(subcommand="asn")
        parser_c = subparsers.add_parser("domain", help="Request info on a domain")
        parser_c.add_argument("DOMAIN", help="DOMAIN")
        parser_c.add_argument(
            "--json", "-j", action="store_true", help="Show raw JSON info"
        )
        parser_c.set_defaults(subcommand="domain")
        self.parser = parser

    def run(self, args, plugins):
        if "subcommand" in args:
            if args.subcommand == "ip":
                r = Robtex()
                res = r.get_ip_info(args.IP)
                if args.json:
                    print(
                        json.dumps(res, sort_keys=True, indent=4, default=json_serial)
                    )
                else:
                    print("AS %i: %s" % (res["as"], res["asname"]))
                    print("Location: %s, %s" % (res["city"], res["country"]))
                    print("BGP Route: %s, %s" % (res["bgproute"], res["routedesc"]))
                    print("Whois: %s" % res["whoisdesc"])
                    print("https://www.robtex.com/ip-lookup/%s" % args.IP)
                    if "pas" in res:
                        if len(res["pas"]):
                            print("Passive DNS:")
                            for d in res["pas"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
                    if "pash" in res:
                        if len(res["pash"]):
                            print("Passive DNS History:")
                            for d in res["pash"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
                    if "act" in res:
                        if len(res["act"]):
                            print("Active DNS:")
                            for d in res["act"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
                    if "acth" in res:
                        if len(res["acth"]):
                            print("ACtive DNS History:")
                            for d in res["acth"]:
                                print("\t%s %s" % (d["date"].isoformat(), d["o"]))
            elif args.subcommand == "asn":
                r = Robtex()
                res = r.get_asn_info(args.ASN)
                if args.json:
                    print(
                        json.dumps(res, sort_keys=True, indent=4, default=json_serial)
                    )
                else:
                    print("ASN Routes:")
                    for n in res["nets"]:
                        print("[+] %s" % n["n"])
            elif args.subcommand == "domain":
                r = Robtex()
                res = r.get_pdns_domain(unbracket(args.DOMAIN))
                if args.json:
                    print(
                        json.dumps(res, sort_keys=True, indent=4, default=json_serial)
                    )
                else:
                    if len(res) == 0:
                        print("No information on this domain")
                    else:
                        print("Passive DNS info:")
                        for r in res:
                            print(
                                "[+] %s\t%s\t(%s -> %s)"
                                % (
                                    r["rrtype"],
                                    r["rrdata"],
                                    r["time_first_o"].isoformat(),
                                    r["time_last_o"].isoformat(),
                                )
                            )
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()

    def intel(self, type, query, data):
        if type == "domain":
            print("[+] Checking Robtex....")
            try:
                rob = Robtex()
                res = rob.get_pdns_domain(query)
                for d in res:
                    if d["rrtype"] in ["A", "AAAA"]:
                        data["passive_dns"].append(
                            {
                                "first": d["time_first_o"].astimezone(pytz.utc),
                                "last": d["time_last_o"].astimezone(pytz.utc),
                                "ip": d["rrdata"],
                                "source": "Robtex",
                            }
                        )
            except RobtexError:
                print("Robtex query failed")
        elif type == "ip":
            print("[+] Checking Robtex....")
            try:
                rob = Robtex()
                res = rob.get_ip_info(query)
                if "pas" in res:
                    if len(res["pas"]):
                        for d in res["pas"]:
                            data["passive_dns"].append(
                                {
                                    "first": d["date"].astimezone(pytz.utc),
                                    "last": "",
                                    "domain": d["o"],
                                    "source": "Robtex",
                                }
                            )
            except RobtexError:
                print("Robtex query failed")



#! /usr/bin/env python
import json
import os
import sys
import time

from harpoon.commands.base import Command
from harpoon.lib.ip2locationio import IP2Locationio, IP2LocationioError
from harpoon.lib.utils import unbracket


class CommandIPInfo(Command):
    """
    # IP2Location.io plugin

    **Query IP2Location.io API**

    * Get info on an IP : `harpoon ip2locationio ip IP`
    * Get infos on a list of IPs in a file : `harpoon ip2locationio file FILE`
    """

    name = "ip2locationio"
    description = "Request IP2Location.io information"
    config = {"IP2Locationio": ["token"]}

    def add_arguments(self, parser):
        subparsers = parser.add_subparsers(help="Subcommand")
        parser_a = subparsers.add_parser("ip", help="Information on an IP")
        parser_a.add_argument("IP", help="IP address")
        parser_a.set_defaults(subcommand="ip")
        parser_b = subparsers.add_parser("file", help="Information on a list of IPs")
        parser_b.add_argument("FILE", help="Filename")
        parser_b.add_argument(
            "--delay",
            "-d",
            type=int,
            default=1,
            help="Delay between two queries in seconds",
        )
        parser_b.set_defaults(subcommand="file")
        self.parser = parser

    def run(self, args, plugins):
        ip2locationio = IP2Locationio(token=self._config_data["IP2Locationio"]["token"])
        if "subcommand" in args:
            if args.subcommand == "ip":
                try:
                    infos = ip2locationio.get_infos(unbracket(args.IP))
                except IP2LocationioError:
                    print("Invalid request")
                else:
                    print(
                        json.dumps(
                            infos, sort_keys=True, indent=4, separators=(",", ": ")
                        )
                    )
            elif args.subcommand == "file":
                if os.path.isfile(args.FILE):
                    with open(args.FILE) as f:
                        data = f.read().split("\n")
                    print(
                        "IP;Domain;City;Region;Country;Location;"
                        + "ISP;ASN;AS Name;"
                        + "Is Proxy;Proxy Type;Threat Type;Last Seen in"
                    )
                    for d in data:
                        if d.strip() == "":
                            continue
                        ip = unbracket(d.strip())
                        try:
                            infos = ip2locationio.get_infos(ip)
                        except IP2LocationioError:
                            print("%s;;;;;;;;;;;;;" % ip)
                        else:
                            loc = str(infos["latitude"]) + "," + str(infos["longitude"])
                            if "proxy" in infos:
                                print(
                                    "%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s"
                                    % (
                                        ip,
                                        infos["domain"] if "domain" in infos else "",
                                        infos["city_name"] if "city" in infos else "",
                                        infos["region_name"]
                                        if "region" in infos
                                        else "",
                                        infos["country_name"]
                                        if "country" in infos
                                        else "",
                                        loc,
                                        infos["isp"] if "isp" in infos else "",
                                        infos["asn"] if "asn" in infos else "",
                                        infos["as"] if "as" in infos else "",
                                        infos["is_proxy"]
                                        if "is_proxy" in infos
                                        else "",
                                        infos["proxy"]["proxy_type"]
                                        if "proxy_type" in infos["proxy"]
                                        else "",
                                        infos["proxy"]["threat"]
                                        if "threat" in infos["proxy"]
                                        else "",
                                        (
                                            str(infos["proxy"]["last_seen"])
                                            + (
                                                " days"
                                                if infos["proxy"]["last_seen"] > 1
                                                else " day"
                                            )
                                        )
                                        if "last_seen" in infos["proxy"]
                                        else "",
                                    )
                                )
                            else:
                                print(
                                    "%s;%s;%s;%s;%s;%s;%s;%s;%s;;;;;"
                                    % (
                                        ip,
                                        infos["domain"] if "domain" in infos else "",
                                        infos["city_name"],
                                        infos["region_name"],
                                        infos["country_name"],
                                        loc,
                                        infos["isp"] if "isp" in infos else "",
                                        infos["asn"] if "asn" in infos else "",
                                        infos["as"] if "as" in infos else "",
                                    )
                                )
                        time.sleep(args.delay)

                else:
                    print("This file does not exist")
                    sys.exit(1)
            else:
                self.parser.print_help()
        else:
            self.parser.print_help()
